# -*- coding: utf-8 -*-

# Copyright(C) 2011-2014 Laurent Bachelier
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


import re

from weboob.tools.capabilities.paste import BasePasteModule
from weboob.tools.backend import Module
from weboob.capabilities.base import NotLoaded

from .browser import PastealaconBrowser, PastealaconPaste


class PastealaconModule(Module, BasePasteModule):
    NAME = 'pastealacon'
    MAINTAINER = u'Laurent Bachelier'
    EMAIL = 'laurent@bachelier.name'
    VERSION = '1.3'
    DESCRIPTION = u'Paste à la con text sharing tool'
    LICENSE = 'AGPLv3+'
    BROWSER = PastealaconBrowser

    EXPIRATIONS = {
        24 * 3600: 'd',
        24 * 3600 * 30: 'm',
        False: 'f',
    }

    def new_paste(self, *args, **kwargs):
        return PastealaconPaste(*args, **kwargs)

    def can_post(self, contents, title=None, public=None, max_age=None):
        try:
            contents.encode('ISO-8859-1')
        except UnicodeEncodeError:
            return 0
        if public is False:
            return 0
        if max_age is not None:
            if self.get_closest_expiration(max_age) is None:
                return 0
        # the "title" is filtered (does not even accepts dots)
        if not title or re.match('^\w+$', title) and len(title) <= 24:
            return 2
        return 1

    def get_paste(self, _id):
        return self.browser.get_paste(_id)

    def fill_paste(self, paste, fields):
        # if we only want the contents
        if fields == ['contents']:
            if paste.contents is NotLoaded:
                contents = self.browser.get_contents(paste.id)
                paste.contents = contents
        # get all fields
        elif fields is None or len(fields):
            self.browser.fill_paste(paste)
        return paste

    def post_paste(self, paste, max_age=None):
        if max_age is not None:
            expiration = self.get_closest_expiration(max_age)
        else:
            expiration = None
        self.browser.post_paste(paste, expiration=self.EXPIRATIONS.get(expiration))

    OBJECTS = {PastealaconPaste: fill_paste}



# -*- coding: utf-8 -*-

# Copyright(C) 2015 Guilhem Bonnefille
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

import re

from weboob.browser.pages import HTMLPage, JsonPage
from weboob.browser.elements import ItemElement, ListElement, method
from weboob.browser.filters.standard import Regexp, Format, CleanText, Duration, DateTime, Filter
from weboob.browser.filters.html import Link, XPath, CleanHTML
from weboob.browser.filters.json import Dict

from weboob.capabilities import NotLoaded
from weboob.capabilities.image import Thumbnail
from weboob.capabilities.collection import Collection

from .video import RmllVideo
from datetime import timedelta

BASE_URL = 'http://video.rmll.info'


class NormalizeThumbnail(Filter):
    def filter(self, thumbnail):
        if not thumbnail.startswith('http'):
            thumbnail = BASE_URL + thumbnail
        if thumbnail == "http://rmll.ubicast.tv/statics/mediaserver/images/video_icon.png":
            # This is the default: remove it as any frontend default should be better
            thumbnail = None
        return thumbnail


class RmllDuration(Duration):
    _regexp = re.compile(r'((?P<hh>\d+) h )?((?P<mm>\d+) m )?(?P<ss>\d+) s')
    kwargs = {'hours': 'hh', 'minutes': 'mm', 'seconds': 'ss'}


def create_video(metadata):
    video = RmllVideo(metadata['oid'])

    video.title = unicode(metadata['title'])
    video.date = DateTime(Dict('creation'), default=NotLoaded)(metadata)
    video.duration = RmllDuration(Dict('duration', default=''), default=NotLoaded)(metadata)
    thumbnail = NormalizeThumbnail(Dict('thumb'))(metadata)
    video.thumbnail = Thumbnail(thumbnail)
    video.thumbnail.url = video.thumbnail.id
    video.url = NotLoaded

    return video


class RmllVideoPage(HTMLPage):
    @method
    class get_video(ItemElement):
        klass = RmllVideo

        obj_id = CleanHTML('/html/head/meta[@property="og:url"]/@content') & CleanText() & Regexp(pattern=r'.*/permalink/(.+)/$')
        obj_title = Format(u'%s', CleanHTML('/html/head/meta[@name="DC.title"]/@content') & CleanText())
        obj_description = Format(u'%s', CleanHTML('/html/head/meta[@property="og:description"]/@content') & CleanText())

        def obj_thumbnail(self):
            url = NormalizeThumbnail(CleanText('/html/head/meta[@property="og:image"]/@content'))(self)
            if url:
                thumbnail = Thumbnail(url)
                thumbnail.url = thumbnail.id
                return thumbnail

        def obj_url(self):
            links = XPath('//div[@id="download_links"]/div[@class="paragraph"]/div[has-class("share")]/a[@target="_blank"]/@href')(self)
            for link in links:
                ext = str(link).split('.')[-1]
                self.logger.debug("Link:%s Ext:%s", link, ext)
                if ext in ['mp4', 'webm']:
                    return self.page.browser.BASEURL + unicode(link)


class RmllDurationPage(JsonPage):
    def get_duration(self):
        return timedelta(seconds=Dict('duration')(self.doc))


class RmllCollectionPage(HTMLPage):

    @method
    class iter_videos(ListElement):
        item_xpath = '//div[@class="item-entry type-video " or @class="item-entry type-vod "]'

        class item(ItemElement):
            klass = RmllVideo

            obj_id = Link('a') & Regexp(pattern=r'.*/videos/(.+)/$')
            obj_title = Format(u'%s', CleanHTML('a/span/span/span[@class="item-entry-title"]') & CleanText())
            obj_url = NotLoaded
            # obj_date = XPath('a/span/span/span[@class="item-entry-creation"]')

            obj_duration = CleanText('a/span/span/span[@class="item-entry-duration"]') & RmllDuration()

            def obj_thumbnail(self):
                thumbnail = NormalizeThumbnail(CleanText('a/span[@class="item-entry-preview"]/img/@src'))(self)
                if thumbnail:
                    thumbnail = Thumbnail(thumbnail)
                    thumbnail.url = thumbnail.id
                    return thumbnail


class RmllChannelsPage(JsonPage):
    def iter_resources(self, split_path):
        if 'channels' in self.doc:
            for metadata in self.doc['channels']:
                collection = Collection(split_path+[metadata['oid']], metadata['title'])
                yield collection

        if 'videos' in self.doc:
            for metadata in self.doc['videos']:
                video = create_video(metadata)
                yield video


class RmllLatestPage(JsonPage):
    def iter_resources(self):
        for metadata in self.doc['items']:
            if metadata['type'] == 'c':
                collection = Collection([metadata['oid']], metadata['title'])
                yield collection

            if metadata['type'] == 'v':
                video = create_video(metadata)
                yield video


class RmllSearchPage(JsonPage):
    def iter_resources(self):
        if 'videos' in self.doc:
            for metadata in self.doc['videos']:
                video = create_video(metadata)
                yield video



# -*- coding: utf-8 -*-

# Copyright(C) 2013 Julien Veyssier
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from weboob.tools.test import BackendTest

from random import choice


class TvsubtitlesTest(BackendTest):
    MODULE = 'tvsubtitles'

    def test_subtitle(self):
        subtitles = list(self.backend.iter_subtitles('fr', 'sopranos'))
        assert (len(subtitles) > 0)
        for subtitle in subtitles:
            assert subtitle.url.startswith('http')

        # get the file of a random sub
        if len(subtitles):
            subtitle = choice(subtitles)
            self.backend.get_subtitle_file(subtitle.id)

    def test_get_subtitle(self):
        subtitles = list(self.backend.iter_subtitles('fr', 'sopranos'))
        assert (len(subtitles) > 0)
        subtitle = choice(subtitles)
        assert self.backend.get_subtitle(subtitle.id)



#!/usr/bin/env python

# Copyright(C) 2017  Vincent A
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from __future__ import print_function

import ast
import fnmatch
import os
import traceback

import lxml.etree
from weboob.browser.filters import standard


class Error(SyntaxError):
    def __init__(self, file, line, message):
        super(Error, self).__init__('%s:%s: %s' % (file, line, message))
        self.file = file
        self.line = line


def do_visits(*funcs):
    def wrapper(self, node):
        for func in funcs:
            func(self, node)
        self.generic_visit(node)
    return wrapper


class Visitor(ast.NodeVisitor):
    def __init__(self, file, *args, **kwargs):
        super(Visitor, self).__init__(*args, **kwargs)
        self.warnings = kwargs.pop('warnings', False)
        self.file = file

        self.filters = []
        self.filters.extend(f for f in dir(standard) if isinstance(getattr(standard, f), type) and issubclass(getattr(standard, f), standard.CleanText))
        self.filters.extend(['Regexp', 'XPath', 'Attr', 'Link'])

        self.element_context = []

    def check_xpath(self, s, lineno):
        try:
            lxml.etree.XPath(s)
        except lxml.etree.XPathSyntaxError as exc:
            raise Error(self.file, lineno, exc)

        if self.warnings:
            if not s.startswith('.') and len(self.element_context) >= 2:
                if self.element_context[-1] == 'ItemElement' and self.element_context[-2] in ('TableElement', 'ListElement'):
                    print('%s:%s: probable missing "." at start of XPath' % (self.file, lineno))

    def _item_xpath(self, node):
        try:
            target, = node.targets
        except ValueError:
            return
        if not isinstance(target, ast.Name) or target.id != 'item_xpath':
            return
        try:
            if self.element_context[-1] not in ('TableElement', 'ListElement'):
                return
        except IndexError:
            return
        if not isinstance(node.value, ast.Str):
            return

        self.check_xpath(node.value.s, node.lineno)

    visit_Assign = do_visits(_item_xpath)

    def _xpath_call(self, node):
        if not isinstance(node.func, ast.Attribute):
            return
        if node.func.attr != 'xpath':
            return
        try:
            if not isinstance(node.args[0], ast.Str):
                return
        except IndexError:
            return

        self.check_xpath(node.args[0].s, node.lineno)

    def _filter_call(self, node):
        if not isinstance(node.func, ast.Name):
            return
        if node.func.id not in self.filters:
            return
        try:
            if not isinstance(node.args[0], ast.Str):
                return
        except IndexError:
            return

        self.check_xpath(node.args[0].s, node.lineno)

    visit_Call = do_visits(_xpath_call, _filter_call)

    def visit_ClassDef(self, node):
        has_element = False

        proceed = False
        for deco in node.decorator_list:
            if isinstance(deco, ast.Name) and deco.id == 'method':
                proceed = True
                break
        if proceed:
            for basenode in node.bases:
                if isinstance(basenode, ast.Name) and basenode.id in ('ListElement', 'ItemElement', 'TableElement'):
                    self.element_context.append(basenode.id)
                    has_element = True
                    break

        self.generic_visit(node)

        if has_element:
            self.element_context.pop()


def search_py(root):
    for path, dirs, files in os.walk(root):
        dirs.sort()
        for f in fnmatch.filter(files, '*.py'):
            yield os.path.join(path, f)


for fn in search_py(os.path.normpath(os.path.dirname(__file__) + '/../modules')):
    with open(fn) as fd:
        try:
            node = ast.parse(fd.read(), fn)
        except SyntaxError as exc:
            print('In file', fn)
            traceback.print_exc(exc)
    try:
        Visitor(fn).visit(node)
    except SyntaxError as exc:
        print(exc)



# -*- coding: utf-8 -*-

# Copyright(C) 2011  Romain Bignon
# Copyright(C) 2012  Benjamin Drieu
#
# This file is *not yet* part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.capabilities.video import CapVideo
from weboob.tools.backend import Module

from .browser import TricTracTVBrowser
from .video import TricTracTVVideo


__all__ = ['TricTracTVModule']


class TricTracTVModule(Module, CapVideo):
    NAME = 'trictractv'
    MAINTAINER = u'Benjamin Drieu'
    EMAIL = 'benjamin@drieu.org'
    VERSION = '1.3'
    DESCRIPTION = u'TricTrac.tv video website'
    LICENSE = 'AGPLv3+'
    BROWSER = TricTracTVBrowser

    def get_video(self, _id):
        with self.browser:
            return self.browser.get_video(_id)

    def search_videos(self, pattern=None, sortby=CapVideo.SEARCH_RELEVANCE, nsfw=False):
        with self.browser:
            return self.browser.search_videos(pattern)

    def fill_video(self, video, fields):
        if fields != ['thumbnail']:
            # if we don't want only the thumbnail, we probably want also every fields
            with self.browser:
                video = self.browser.get_video(TricTracTVVideo.id2url(video.id), video)
        if 'thumbnail' in fields and video.thumbnail:
            with self.browser:
                video.thumbnail.data = self.browser.readurl(video.thumbnail.url)

        return video

    OBJECTS = {TricTracTVVideo: fill_video}



# -*- coding: utf-8 -*-

# Copyright(C) 2013-2014      Florent Fourcot
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from weboob.capabilities.parcel import Event, ParcelNotFound
from weboob.browser import PagesBrowser, URL
from weboob.browser.elements import ItemElement, ListElement, method
from weboob.browser.filters.standard import CleanText, Date
from weboob.browser.pages import HTMLPage
from weboob.browser.profiles import Firefox


__all__ = ['ColissimoBrowser']


class TrackingPage(HTMLPage):
    @method
    class iter_infos(ListElement):
        item_xpath = '//div[has-class("results-suivi")]//table/tbody/tr'

        class item(ItemElement):
            klass = Event

            obj_date = Date(CleanText('td[1]'), dayfirst=True)
            obj_activity = CleanText('td[2]')
            obj_location = CleanText('td[3]')

    def get_error(self):
        return CleanText('//div[has-class("error-suivi")]')(self.doc)


class ColissimoBrowser(PagesBrowser):
    BASEURL = 'http://www.laposte.fr'
    PROFILE = Firefox()

    tracking_url = URL('/particulier/outils/suivre-vos-envois\?code=(?P<_id>.*)', TrackingPage)

    def get_tracking_info(self, _id):
        self.tracking_url.stay_or_go(_id=_id)
        events = list(self.page.iter_infos())
        if len(events) == 0:
            error = self.page.get_error()
            raise ParcelNotFound(u"Parcel not found: {}".format(error))
        return events



# -*- coding: utf-8 -*-

# Copyright(C) 2012-2016 Roger Philibert
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from weboob.browser import LoginBrowser, URL
from weboob.exceptions import BrowserIncorrectPassword
from weboob.tools.json import json


__all__ = ['OkCBrowser']


def need_login(func):
    def inner(browser, *args, **kwargs):
        if not browser.access_token:
            browser.do_login()
        return func(browser, *args, **kwargs)
    return inner


class OkCBrowser(LoginBrowser):
    BASEURL = 'https://www.okcupid.com'

    login = URL('/login')
    threads = URL('/messages')
    messages = URL('/apitun/messages/conversations/global_messaging')
    thread_delete = URL(r'/1/apitun/messages/conversations/(?P<thread_id>\d+)/delete')
    message_send = URL('/apitun/messages/send')
    quickmatch = URL(r'/quickmatch\?okc_api=1')
    like = URL(r'/1/apitun/profile/(?P<user_id>\d+)/like')
    profile = URL(r'/apitun/profile/(?P<user_id>\d+)')
    full_profile = URL(r'/profile/(?P<username>.*)\?okc_api=1')

    access_token = None
    me = None

    def do_login(self):
        r = self.login.go(data={'username': self.username, 'password': self.password, 'okc_api': 1}).json()

        if not 'oauth_accesstoken' in r:
            raise BrowserIncorrectPassword(r['status_str'])

        self.access_token = r['oauth_accesstoken']
        self.me = {'userid':    r['userid'],
                   'username':  r['screenname'],
                  }
        self.session.headers['X-OkCupid-Platform'] = 'DESKTOP'
        self.session.headers['X-Requested-With'] = 'XMLHttpRequest'
        self.session.headers['Authorization'] = 'Bearer %s' % self.access_token

    @need_login
    def get_threads_list(self, folder=1):
        return self.threads.go(params={'okc_api': 1, 'folder': folder, 'messages_dropdown_ajax': 1}).json()

    @need_login
    def get_thread_messages(self, thread_id):
        r = self.messages.go(params={'access_token': self.access_token,
                                     '_json': '{"userids":["%s"]}' % thread_id}).json()
        return r[thread_id]

    @need_login
    def post_message(self, thread_id, content):
        data = {'body': content,
                'profile_tab': '',
                'receiverid': thread_id,
                'service': 'mailbox',
                'source': 'desktop_global'}

        self.message_send.go(params={'access_token': self.access_token},
                             data=json.dumps(data))

    @need_login
    def delete_thread(self, thread_id):
        self.thread_delete.go(method='POST', thread_id=thread_id)

    @need_login
    def find_match_profile(self):
        r = self.quickmatch.go().json()
        return r['tracking_userid']

    @need_login
    def do_rate(self, user_id):
        self.like.go(method='POST', user_id=user_id)

    @need_login
    def get_username(self, user_id):
        return self.profile.go(user_id=user_id).json()['username']

    @need_login
    def get_profile(self, username):
        if username.isdigit():
            username = self.get_username(username)

        return self.full_profile.go(username=username).json()



# -*- coding: utf-8 -*-

# Copyright(C) 2014 Romain Bignon
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

import json
import re
from datetime import date
from decimal import Decimal

from weboob.capabilities.base import NotAvailable
from weboob.capabilities.bank import Account
from weboob.tools.capabilities.bank.transactions import FrenchTransaction, sorted_transactions
from weboob.exceptions import BrowserIncorrectPassword, BrowserHTTPError, BrowserUnavailable, ActionNeeded, ParseError
from weboob.browser import DomainBrowser


__all__ = ['BredBrowser']


class Transaction(FrenchTransaction):
    PATTERNS = [(re.compile('^.*Virement (?P<text>.*)'), FrenchTransaction.TYPE_TRANSFER),
                (re.compile(u'PRELEV SEPA (?P<text>.*)'),        FrenchTransaction.TYPE_ORDER),
                (re.compile(u'.*Prélèvement.*'),        FrenchTransaction.TYPE_ORDER),
                (re.compile(u'^(REGL|Rgt)(?P<text>.*)'),        FrenchTransaction.TYPE_ORDER),
                (re.compile('^(?P<text>.*) Carte \d+\s+ LE (?P<dd>\d{2})/(?P<mm>\d{2})/(?P<yy>\d{2})'),
                                                          FrenchTransaction.TYPE_CARD),
                (re.compile(u'^Débit mensuel.*'), FrenchTransaction.TYPE_CARD_SUMMARY),
                (re.compile(u"^Retrait d'espèces à un DAB (?P<text>.*) CARTE [X\d]+ LE (?P<dd>\d{2})/(?P<mm>\d{2})/(?P<yy>\d{2})"),
                                                          FrenchTransaction.TYPE_WITHDRAWAL),
                (re.compile(u'^Paiement de chèque (?P<text>.*)'),  FrenchTransaction.TYPE_CHECK),
                (re.compile(u'^(Cotisation|Intérêts) (?P<text>.*)'), FrenchTransaction.TYPE_BANK),
                (re.compile(u'^(Remise Chèque|Remise de chèque)\s*(?P<text>.*)'), FrenchTransaction.TYPE_DEPOSIT),
                (re.compile('^Versement (?P<text>.*)'), FrenchTransaction.TYPE_DEPOSIT),
               ]


class BredBrowser(DomainBrowser):
    BASEURL = 'https://www.bred.fr'

    def __init__(self, accnum, login, password, *args, **kwargs):
        super(BredBrowser, self).__init__(*args, **kwargs)
        self.login = login
        # Bred only use first 8 char (even if the password is set to be bigger)
        # The js login form remove after 8th char. No comment.
        self.password = password[:8]
        self.accnum = accnum
        self.universes = None
        self.current_univers = None

    def do_login(self, login, password):
        r = self.open('/transactionnel/Authentication', data={'identifiant': login, 'password': password})

        if 'gestion-des-erreurs/erreur-pwd' in r.url:
            raise BrowserIncorrectPassword('Bad login/password.')
        if 'gestion-des-erreurs/opposition' in r.url:
            raise BrowserIncorrectPassword('Your account is disabled')
        if '/pages-gestion-des-erreurs/erreur-technique' in r.url:
            raise BrowserUnavailable('A technical error occured')
        if '/pages-gestion-des-erreurs/message-tiers-oppose' in r.url:
            raise ActionNeeded('Cannot connect to account because 2-factor authentication is enabled')

    ACCOUNT_TYPES = {'000': Account.TYPE_CHECKING,
                     '999': Account.TYPE_MARKET,
                     '011': Account.TYPE_CARD,
                     '023': Account.TYPE_SAVINGS,
                     '078': Account.TYPE_SAVINGS,
                     '080': Account.TYPE_SAVINGS,
                     '027': Account.TYPE_SAVINGS,
                     '037': Account.TYPE_SAVINGS,
                     '730': Account.TYPE_DEPOSIT,
                    }

    def api_open(self, *args, **kwargs):
        try:
            return super(BredBrowser, self).open(*args, **kwargs)
        except BrowserHTTPError:
            self.do_login(self.login, self.password)
            return super(BredBrowser, self).open(*args, **kwargs)

    def set_universes(self):
        universes = []
        r = self.api_open('/transactionnel/services/applications/menu/getMenuUnivers')
        for univers in r.json()['content']['menus']:
            universes.append(univers['universKey'])
        if not universes:
            # There is just the default univers here.
            universes.append('')
            self.current_univers = ''
        else:
            # The following is needed to get the default univers in the list.
            self.move_to_univers(universes[0])
            r = self.api_open('/transactionnel/services/applications/menu/getMenuUnivers')
            for univers in r.json()['content']['menus']:
                if univers['universKey'] not in universes:
                    universes.append(univers['universKey'])
        self.universes = universes

    def move_to_univers(self, univers):
        x_token_bred = self.api_open('/transactionnel/services/rest/User/nonce?random=').json()['content']
        data = {}
        data['all'] = 'true'
        data['univers'] = univers
        self.api_open('/transactionnel/services/rest/User/switch', data=json.dumps(data), headers={'x-token-bred': x_token_bred})
        self.current_univers = univers

    def get_accounts_list(self):
        if not self.universes:
            self.set_universes()
        accounts = []
        for univers in self.universes:
            if univers != self.current_univers:
                self.move_to_univers(univers)
            accounts.extend(self.get_list())
            accounts.extend(self.get_loans_list())
        return accounts

    def get_loans_list(self):
        r = self.api_open('/transactionnel/services/applications/prets/liste')

        if 'content' in r.json():
            for content in r.json()['content']:
                a = Account()
                a.id = "%s.%s" % (content['comptePrets'].strip(), content['numeroDossier'].strip())
                a.type = Account.TYPE_LOAN
                a.label = ' '.join([content['intitule'].strip(), content['libellePrets'].strip()])
                a.balance = -Decimal(str(content['montantCapitalDu']['valeur']))
                a.currency = content['montantCapitalDu']['monnaie']['code'].strip()
                yield a

    def get_list(self):
        r = self.api_open('/transactionnel/services/rest/Account/accounts')

        seen = set()

        for content in r.json()['content']:
            if self.accnum != '00000000000' and content['numero'] != self.accnum:
                continue
            for poste in content['postes']:
                a = Account()
                a._number = content['numeroLong']
                a._nature = poste['codeNature']
                a._consultable = poste['consultable']
                a._univers = self.current_univers
                a.id = '%s.%s' % (a._number, a._nature)

                if a.id in seen:
                    # some accounts like "compte à terme fidélis" have the same _number and _nature
                    # but in fact are kind of closed, so worthless...
                    self.logger.warning('ignored account id %r (%r) because it is already used', a.id, poste.get('numeroDossier'))
                    continue
                seen.add(a.id)

                a.type = self.ACCOUNT_TYPES.get(poste['codeNature'], Account.TYPE_UNKNOWN)
                if a.type == Account.TYPE_CHECKING:
                    iban_response = self.api_open('/transactionnel/services/rest/Account/account/%s/iban' % a._number).json()
                    a.iban = iban_response['content']['iban'] if 'content' in iban_response else NotAvailable
                else:
                    a.iban = NotAvailable

                if 'numeroDossier' in poste and poste['numeroDossier']:
                    a._file_number = poste['numeroDossier']
                    a.id += '.%s' % a._file_number

                if poste['postePortefeuille']:
                    a.label = u'Portefeuille Titres'
                    a.balance = Decimal(str(poste['montantTitres']['valeur']))
                    a.currency = poste['montantTitres']['monnaie']['code'].strip()
                    if not a.balance and not a.currency and 'dateTitres' not in poste:
                        continue
                    yield a

                if 'libelle' not in poste:
                    continue

                a.label = ' '.join([content['intitule'].strip(), poste['libelle'].strip()])
                a.balance = Decimal(str(poste['solde']['valeur']))
                a.currency = poste['solde']['monnaie']['code'].strip()
                # Some accounts may have balance currency
                if 'Solde en devises' in a.label and a.currency != u'EUR':
                    a.id += str(poste['monnaie']['codeSwift'])
                yield a

    def get_history(self, account, coming=False):
        if account.type is Account.TYPE_LOAN or not account._consultable:
            raise NotImplementedError()

        if account._univers != self.current_univers:
            self.move_to_univers(account._univers)

        seen = set()
        offset = 0
        next_page = True
        while next_page:
            r = self.api_open('/transactionnel/services/applications/operations/get/%(number)s/%(nature)s/00/%(currency)s/%(startDate)s/%(endDate)s/%(offset)s/%(limit)s' %
                          {'number': account._number,
                           'nature': account._nature,
                           'currency': account.currency,
                           'startDate': '2000-01-01',
                           'endDate': date.today().strftime('%Y-%m-%d'),
                           'offset': offset,
                           'limit': 50
                          })
            transactions = []
            for op in reversed(r.json()['content']['operations']):
                t = Transaction()
                t.id = op['id']
                if op['id'] in seen:
                    raise ParseError('There are several transactions with the same ID, probably an infinite loop')

                seen.add(t.id)
                d = date.fromtimestamp(op.get('dateDebit', op.get('dateOperation'))/1000)
                op['details'] = [re.sub('\s+', ' ', i).replace('\x00', '') for i in op['details'] if i] # sometimes they put "null" elements...
                label = re.sub('\s+', ' ', op['libelle']).replace('\x00', '')
                raw = ' '.join([label] + op['details'])
                vdate = date.fromtimestamp(op.get('dateValeur', op.get('dateDebit', op.get('dateOperation')))/1000)
                t.parse(d, raw, vdate=vdate)
                t.amount = Decimal(str(op['montant']))
                t.rdate = date.fromtimestamp(op.get('dateOperation', op.get('dateDebit'))/1000)
                if 'categorie' in op:
                    t.category = op['categorie']
                t.label = label
                t._coming = op['intraday']
                if t._coming:
                    # coming transactions have a random uuid id (inconsistent between requests)
                    t.id = ''
                transactions.append(t)

            # Transactions are unsorted
            for t in sorted_transactions(transactions):
                if coming == t._coming:
                    yield t
                elif coming and not t._coming:
                    # coming transactions are at the top of history
                    self.logger.debug('stopping coming after %s', t)
                    return

            next_page = bool(transactions)
            offset += 50

            assert offset < 10000, 'the site may be doing an infinite loop'




"ArticlePage object for presseurope"
# -*- coding: utf-8 -*-

# Copyright(C) 2012 Florent Fourcot
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from weboob.browser.pages import AbstractPage
from weboob.browser.filters.html import CSS, CleanHTML


class PresseuropPage(AbstractPage):
    "PresseuropPage object for presseurop"
    _selector = CSS
    PARENT = 'genericnewspaper'
    PARENT_URL = 'generic_news_page'

    def on_loaded(self):
        self.main_div = self.doc.getroot()
        self.element_title_selector = "title"
        self.element_author_selector = "a[rel=author], div.profilecartoontext>p>a"
        self.element_body_selector = "div.block, div.panel, div.bodytext"

    def get_body(self):
        element_body = self.get_element_body()
        self.try_drop_tree(element_body, "li.button-social")
        self.try_drop_tree(element_body, "div.sharecount")
        self.try_drop_tree(element_body, "p.ruledtop")
        self.try_drop_tree(element_body, "p.ctafeedback")
        self.try_drop_tree(element_body, "aside.articlerelated")
        self.try_drop_tree(element_body, "div.sharecount")
        self.try_drop_tree(element_body, "iframe")
        self.clean_relativ_urls(element_body, "http://presseurop.eu")
        return CleanHTML('.')(element_body)

    def get_title(self):
        title = super(self.__class__, self).get_title()
        title = title.split('|')[0]
        return title

    def get_author(self):
        author = super(self.__class__, self).get_author()
        try:
            source = self.doc.getroot().xpath(
                    "//span[@class='sourceinfo']/a")[0]
            source = source.text
            author = author + " | " + source
            return author
        except:
            return author

    def get_daily_date(self):
        plink = self.doc.getroot().xpath("//p[@class='w200']")
        if len(plink) > 0:
            link = plink[0].xpath('a')[0]
            date = link.attrib['href'].split('/')[3]
            return date
        return None



# -*- coding: utf-8 -*-

# Copyright(C) 2013      Laurent Bachelier
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from logging import error
import re
from io import BytesIO

from weboob.browser.pages import HTMLPage, LoggedPage
from weboob.browser.elements import ListElement, ItemElement, method
from weboob.browser.filters.standard import CleanText, CleanDecimal, Date, Env
from weboob.browser.filters.html import Attr
from weboob.tools.capabilities.bank.transactions import FrenchTransaction
from weboob.capabilities.profile import Profile, Person
from weboob.exceptions import ActionNeeded
from weboob.tools.json import json

from weboob.capabilities.base import NotAvailable

from ..captcha import Captcha, TileError


class Transaction(FrenchTransaction):
    PATTERNS = [(re.compile(r'^CARTE \w+ RETRAIT DAB.* (?P<dd>\d{2})/(?P<mm>\d{2})( (?P<HH>\d+)H(?P<MM>\d+))? (?P<text>.*)'),
                                                            FrenchTransaction.TYPE_WITHDRAWAL),
                (re.compile(r'^CARTE \w+ (?P<dd>\d{2})/(?P<mm>\d{2})( A (?P<HH>\d+)H(?P<MM>\d+))? RETRAIT DAB (?P<text>.*)'),
                                                            FrenchTransaction.TYPE_WITHDRAWAL),
                (re.compile(r'^CARTE \w+ REMBT (?P<dd>\d{2})/(?P<mm>\d{2})( A (?P<HH>\d+)H(?P<MM>\d+))? (?P<text>.*)'),
                                                            FrenchTransaction.TYPE_PAYBACK),
                (re.compile(r'^DEBIT MENSUEL CARTE.*'),
                                                            FrenchTransaction.TYPE_CARD_SUMMARY),
                (re.compile(r'^(?P<category>CARTE) \w+ (?P<dd>\d{2})/(?P<mm>\d{2}) (?P<text>.*)'),
                                                            FrenchTransaction.TYPE_CARD),
                (re.compile(r'^(?P<dd>\d{2})(?P<mm>\d{2})/(?P<text>.*?)/?(-[\d,]+)?$'),
                                                            FrenchTransaction.TYPE_CARD),
                (re.compile(r'^REMISE CB /(?P<dd>\d{2})/(?P<mm>\d{2}) (?P<text>.*?)/?(-[\d,]+)?$'),
                                                            FrenchTransaction.TYPE_CARD),
                (re.compile(r'^(?P<category>(COTISATION|PRELEVEMENT|TELEREGLEMENT|TIP|PRLV)) (?P<text>.*)'),
                                                            FrenchTransaction.TYPE_ORDER),
                (re.compile(r'^(\d+ )?VIR (PERM )?POUR: (.*?) (REF: \d+ )?MOTIF: (?P<text>.*)'),
                                                            FrenchTransaction.TYPE_TRANSFER),
                (re.compile(r'^(?P<category>VIR(EMEN)?T? \w+) (?P<text>.*)'),
                                                            FrenchTransaction.TYPE_TRANSFER),
                (re.compile(r'^(CHEQUE) (?P<text>.*)'),     FrenchTransaction.TYPE_CHECK),
                (re.compile(r'^(FRAIS) (?P<text>.*)'),      FrenchTransaction.TYPE_BANK),
                (re.compile(r'^(?P<category>ECHEANCEPRET)(?P<text>.*)'),
                                                            FrenchTransaction.TYPE_LOAN_PAYMENT),
                (re.compile(r'^(?P<category>REMISE CHEQUES)(?P<text>.*)'),
                                                            FrenchTransaction.TYPE_DEPOSIT),
                (re.compile(r'^CARTE RETRAIT (?P<text>.*)'),
                                                            FrenchTransaction.TYPE_WITHDRAWAL),
               ]
    _coming = False


class SGPEPage(HTMLPage):
    def get_error(self):
        err = self.doc.getroot().cssselect('div.ngo_mire_reco_message') \
            or self.doc.getroot().cssselect('#nge_zone_centre .nge_cadre_message_utilisateur') \
            or self.doc.xpath(u'//div[contains(text(), "Echec de connexion à l\'espace Entreprises")]') \
            or self.doc.xpath(u'//div[contains(@class, "waitAuthJetonMsg")]')
        if err:
            return err[0].text.strip()


class ChangePassPage(SGPEPage):
    def on_load(self):
        message = (CleanText('//div[@class="ngo_gao_message_intro"]')(self.doc)
                or CleanText('//div[@class="ngo_gao_intro"]')(self.doc)
                or u'Informations manquantes sur le site Société Générale')
        raise ActionNeeded(message)


class LoginPage(SGPEPage):
    def login(self, login, password):
        infos_data = self.browser.open('/sec/vk/gen_crypto?estSession=0').text
        infos_data = re.match('^_vkCallback\((.*)\);$', infos_data).group(1)
        infos = json.loads(infos_data.replace("'", '"'))

        url = '/sec/vk/gen_ui?modeClavier=0&cryptogramme=' + infos["crypto"]
        img = Captcha(BytesIO(self.browser.open(url).content), infos)

        try:
            img.build_tiles()
        except TileError as err:
            error("Error: %s" % err)
            if err.tile:
                err.tile.display()

        form = self.get_form(name=self.browser.LOGIN_FORM)
        form['user_id'] = login
        form['codsec'] = img.get_codes(password[:6])
        form['cryptocvcs'] = infos['crypto']
        form['vk_op'] = 'auth'
        form.url = '/authent.html'
        try:
            form.pop('button')
        except KeyError:
            pass
        form.submit()


class CardsPage(LoggedPage, SGPEPage):
    def get_coming_list(self):
        coming_list = []
        for a in self.doc.xpath('//a[contains(@onclick, "changeCarte")]'):
            m = re.findall("'([^']+)'", Attr(a.xpath('.'), 'onclick')(self))
            params = {}
            params['carte'] = m[1]
            params['date'] = m[2]
            coming_list.append(params)
        return coming_list


class CardHistoryPage(LoggedPage, SGPEPage):
    @method
    class iter_transactions(ListElement):
        item_xpath = '//table[@id="tab-corps"]//tr'

        class item(ItemElement):
            klass = Transaction

            obj_rdate = Date(CleanText('./td[1]'), dayfirst=True)
            obj_date = Date(Env('date'), dayfirst=True, default=NotAvailable)
            obj_raw = Transaction.Raw(CleanText('./td[2]'))
            obj_type = Transaction.TYPE_DEFERRED_CARD
            obj__coming = True
            obj_nopurge = True

            def obj_amount(self):
                return CleanDecimal('./td[3]', replace_dots=True, default=NotAvailable)(self)  \
                    or CleanDecimal('./td[2]', replace_dots=True)(self)

            def condition(self):
                return CleanText('./td[2]')(self)

    def has_next(self):
        current = None
        total = None
        for script in self.doc.xpath('//script'):
            if script.text is None:
                continue

            m = re.search('var pageActive\s+= (\d+)', script.text)
            if m:
                current = int(m.group(1))
            m = re.search("var nombrePage\s+= (\d+)", script.text)
            if m:
                total = int(m.group(1))

        if all((current, total)) and current < total:
            return True

        return False


class ProfileProPage(LoggedPage, SGPEPage):
    @method
    class get_profile(ItemElement):
        klass = Profile

        obj_email = Attr('//input[contains(@name, "_email")]', 'value')

        def obj_name(self):
            civility = CleanText('//td[input[contains(@name, "civilite")][@checked]]/label', default=None)(self) or \
                       CleanText(u'//tr[td[contains(text(), "Civilité")]]/td[last()]')(self)
            firstname = Attr('//input[contains(@name, "_prenom")]', 'value', default=None)(self) or \
                        CleanText(u'//tr[td[contains(text(), "Prénom")]]/td[last()]')(self)
            lastname = Attr('//input[contains(@name, "_nom")]', 'value', default=None)(self) or \
                        CleanText(u'//tr[td[contains(text(), "Nom")]]/td[last()]')(self)
            return "%s %s %s" % (civility, firstname, lastname)


class ProfileEntPage(LoggedPage, SGPEPage):
    @method
    class get_profile(ItemElement):
        klass = Person

        obj_email = Attr('//input[@name="email"]', 'value')
        obj_phone = Attr('//input[@name="telephone"]', 'value')
        obj_job = CleanText('//select[@name="fonction"]/option[@selected]')
        obj_company_name = CleanText(u'//tr[th[contains(text(), "Raison sociale")]]/td')
        obj_company_siren = CleanText(u'//tr[th[contains(text(), "SIREN")]]/td')

        def obj_name(self):
            civility = Attr('//input[@name="civilite"][@checked]', 'value', default=None)(self) or \
                       CleanText(u'//tr[th[contains(text(), "Civilité")]]/td')(self)
            firstname = Attr('//input[@name="prenom"]', 'value', default=None)(self) or \
                        CleanText(u'//tr[th[contains(text(), "Prénom")]]/td')(self)
            lastname = Attr('//input[@name="nom"]', 'value', default=None)(self) or \
                       CleanText(u'//tr[th[contains(text(), "Nom")]]/td')(self)
            return "%s %s %s" % (civility, firstname, lastname)



# -*- coding: utf-8 -*-

# Copyright(C) 2012  Florent Fourcot
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

import StringIO

from mechanize import TextControl

from weboob.deprecated.browser import Page


class LoginPage(Page):
    def on_loaded(self):
        pass

    def login(self, login, password):
        form = list(self.browser.forms())[0]
        self.browser.select_form("aspnetForm")
        self.browser.set_all_readonly(False)
        self.browser.controls.append(TextControl('text', '__ASYNCPOST', {'value': "true"}))
        self.browser['__EVENTTARGET'] = "ctl00$cMain$lnkValider"
        self.browser['ctl00$cMain$ascSaisieMsIsdn$txtMsIsdn'] = login.encode('iso-8859-1')
        self.browser['ctl00$cMain$txtMdp'] = password.encode('iso-8859-1')
        self.browser.submit(nologin=True)
        return form

    def iswait(self):
        spanwait = self.document.xpath('//span[@id="ctl00_ascAttente_timerAttente"]')
        return len(spanwait) > 0

    def iserror(self):
        error = self.document.xpath('//span[@id="ctl00_cMain_ascLibErreur_lblErreur"]')
        return len(error) > 0

    def getredirect(self):
        string = StringIO.StringIO()
        self.document.write(string)
        try:
            redirect = string.getvalue().split('pageRedirect')[1].split('|')[2]
        except:
            redirect = ''
        return redirect

    def next(self, login, form):
        self.browser.form = form
        string = StringIO.StringIO()
        self.document.write(string)
        controlvalue = string.getvalue().split('__EVENTVALIDATION')[1].split('|')[1]
        state = string.getvalue().split('__VIEWSTATE')[1].split('|')[1]
        self.browser.controls.append(TextControl('text', 'ctl00$objScriptManager', {'value': "ctl00$ascAttente$panelAttente|ctl00$ascAttente$timerAttente"}))
        self.browser['__VIEWSTATE'] = state
        self.browser['__EVENTTARGET'] = "ctl00$ascAttente$timerAttente"
        self.browser['__EVENTVALIDATION'] = controlvalue
        self.browser['ctl00$cMain$ascSaisieMsIsdn$txtMsIsdn'] = login.encode('iso-8859-1')
        self.browser['ctl00$cMain$txtMdp'] = ""
        self.browser.submit(nologin=True)



# -*- coding: utf-8 -*-

# Copyright(C) 2013 Julien Veyssier
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from __future__ import print_function

from PyQt5.QtCore import Qt, pyqtSlot as Slot
from PyQt5.QtWidgets import QFrame, QFileDialog

from weboob.applications.qcineoob.ui.subtitle_ui import Ui_Subtitle
from weboob.capabilities.base import empty


class Subtitle(QFrame):
    def __init__(self, subtitle, backend, parent=None):
        super(Subtitle, self).__init__(parent)
        self.parent = parent
        self.backend = backend
        self.ui = Ui_Subtitle()
        self.ui.setupUi(self)

        self.ui.downloadButton.clicked.connect(self.download)

        self.subtitle = subtitle
        self.ui.idEdit.setText(u'%s@%s' % (subtitle.id, backend.name))
        self.ui.nameLabel.setText(u'%s' % subtitle.name)
        if not empty(subtitle.nb_cd):
            self.ui.nbcdLabel.setText(u'%s' % subtitle.nb_cd)
        else:
            self.ui.nbcdLabel.parent().hide()
        if not empty(subtitle.language):
            self.ui.langLabel.setText(u'%s' % subtitle.language)
        else:
            self.ui.langLabel.parent().hide()
        if not empty(subtitle.description):
            self.ui.descriptionPlain.setPlainText(u'%s' % subtitle.description)
        else:
            self.ui.descriptionPlain.parent().hide()
        if not empty(subtitle.url):
            self.ui.urlEdit.setText(u'%s' % subtitle.url)
        else:
            self.ui.downloadButton.setDisabled(True)
            self.ui.downloadButton.setText('Impossible to download this subtitle')

        self.ui.verticalLayout.setAlignment(Qt.AlignTop)

    @Slot()
    def download(self):
        if not empty(self.subtitle.url):
            if self.subtitle.url.endswith('.rar'):
                ext = '.rar'
            elif self.subtitle.url.endswith('.srt'):
                ext = '.srt'
            else:
                ext = '.zip'
        fileDial = QFileDialog(self, 'Save "%s" subtitle file' %
                               self.subtitle.name, '%s%s' % (self.subtitle.name, ext), 'all files (*)')
        fileDial.setAcceptMode(QFileDialog.AcceptSave)
        fileDial.setLabelText(QFileDialog.Accept, 'Save subtitle file')
        fileDial.setLabelText(QFileDialog.FileName, 'Subtitle file name')
        ok = (fileDial.exec_() == 1)
        if not ok:
            return
        result = fileDial.selectedFiles()
        if len(result) > 0:
            dest = result[0]
            data = self.backend.get_subtitle_file(self.subtitle.id)
            try:
                with open(dest, 'w') as f:
                    f.write(data)
            except IOError as e:
                print('Unable to write subtitle file in "%s": %s' % (dest, e), file=self.stderr)
                return 1
            return



# -*- coding: utf-8 -*-

# Copyright(C) 2016      Edouard Lambert
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.tools.backend import Module, BackendConfig
from weboob.tools.value import Value, ValueBackendPassword
from weboob.capabilities.bank import CapBank, AccountNotFound
from weboob.capabilities.base import find_object

from .browser import EsaliaBrowser, CapeasiBrowser, ErehsbcBrowser, BnppereBrowser


__all__ = ['S2eModule']


class S2eModule(Module, CapBank):
    NAME = 's2e'
    DESCRIPTION = u'Épargne Salariale'
    MAINTAINER = u'Edouard Lambert'
    EMAIL = 'elambert@budget-insight.com'
    LICENSE = 'AGPLv3+'
    VERSION = '1.3'

    CONFIG = BackendConfig(
             ValueBackendPassword('login',    label='Identifiant', masked=False),
             ValueBackendPassword('password', label='Code secret', regexp='^(\d{6})$'),
             ValueBackendPassword('secret',   label=u'Réponse secrète (optionnel)', default=''),
             Value('website', label='Banque', default='', choices={'esalia': u'Esalia',
                                                                   'capeasi': u'Capeasi',
                                                                   'erehsbc': u'ERE HSBC',
                                                                   'bnppere': u'BNPP ERE'}))

    BROWSERS = {
        'esalia':  EsaliaBrowser,
        'capeasi': CapeasiBrowser,
        'erehsbc': ErehsbcBrowser,
        'bnppere': BnppereBrowser,
    }

    def create_default_browser(self):
        self.BROWSER = self.BROWSERS[self.config['website'].get()]
        return self.create_browser(self.config['login'].get(),
                                   self.config['password'].get(),
                                   secret=self.config['secret'].get())

    def get_account(self, _id):
        return find_object(self.browser.iter_accounts(), id=_id, error=AccountNotFound)

    def iter_accounts(self):
        return self.browser.iter_accounts()

    def iter_history(self, account):
        return self.browser.iter_history(account)

    def iter_investment(self, account):
        return self.browser.iter_investment(account)

    def iter_pocket(self, account):
        return self.browser.iter_pocket(account)



# -*- coding: utf-8 -*-

# Copyright(C) 2017      Vincent A
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

from weboob.tools.test import BackendTest


class LarousseTest(BackendTest):
    MODULE = 'larousse'

    def _check_translations(self, src, dst, text):
        res = list(self.backend.translate(src, dst, text))
        for trans in res:
            self.assertEqual(trans.lang_src, src)
            self.assertEqual(trans.lang_dst, dst)
        return [t.text for t in res]

    def test_translate(self):
        res = self._check_translations('French', 'English', 'maison')
        assert res
        assert any(t == 'house' for t in res)

        res = self._check_translations('German', 'French', 'kaffee')
        assert res
        assert any('café' in t for t in res)



# -*- coding: utf-8 -*-

# Copyright(C) 2012 Romain Bignon
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.browser.pages import XMLPage, JsonPage, pagination
from weboob.browser.elements import ItemElement, ListElement, DictElement, method
from weboob.browser.filters.json import Dict
from weboob.browser.filters.html import XPath
from weboob.browser.filters.standard import CleanText, CleanDecimal, DateTime, Format, Regexp
from weboob.capabilities.base import NotAvailable
from weboob.capabilities.housing import Housing, HousingPhoto, City, UTILITIES
from weboob.tools.capabilities.housing.housing import PricePerMeterFilter


class CitiesPage(JsonPage):
    @method
    class iter_cities(DictElement):
        item_xpath = '*/values'
        ignore_duplicate = True

        class item(ItemElement):
            klass = City

            def condition(self):
                return (
                    Dict('value', default=None)(self) and
                    Dict('name', default=None)(self) == "ci"
                )

            obj_id = Dict('value')
            obj_name = Dict('label')


class SeLogerItem(ItemElement):
    klass = Housing

    obj_id = CleanText('idAnnonce')
    obj_title = Format(
        "%s %s%s - %s",
        CleanText('titre'),
        CleanText('surface'),
        CleanText('surfaceUnite'),
        CleanText('ville'),
    )
    obj_date = DateTime(CleanText('dtFraicheur'))
    obj_cost = CleanDecimal('prix')

    obj_currency = Regexp(CleanText('prixUnite'),
                          '.*([%s%s%s])' % (u'€', u'$', u'£'), default=u'€')

    obj_area = CleanDecimal('surface', default=NotAvailable)
    obj_price_per_meter = PricePerMeterFilter()
    obj_text = CleanText('descriptif')
    obj_rooms = CleanDecimal('nbPiece|nbPieces', default=NotAvailable)
    obj_bedrooms = CleanDecimal('nbChambre|nbChambres', default=NotAvailable)

    def obj_location(self):
        location = CleanText('adresse', default="")(self)
        quartier = CleanText('quartier', default=None)(self)
        if not location and quartier is not None:
            location = quartier
        ville = CleanText('ville')(self)
        cp = CleanText('cp')(self)
        return u'%s %s (%s)' % (location, ville, cp)

    obj_station = CleanText('proximite', default=NotAvailable)
    obj_url = CleanText('permaLien')


class SearchResultsPage(XMLPage):
    @pagination
    @method
    class iter_housings(ListElement):
        item_xpath = "//annonce"

        def next_page(self):
            page = CleanText('//pageSuivante', default=None, replace=[('http://ws.seloger.com/', '')])(self)
            if page:
                return page

        class item(SeLogerItem):
            def obj_photos(self):
                photos = []

                for photo in XPath('./photos/photo/stdUrl')(self):
                    photos.append(HousingPhoto(CleanText('.')(photo)))

                return photos

            def obj_utilities(self):
                currency = CleanText('prixUnite')(self)
                if "+ch" in currency:
                    return UTILITIES.EXCLUDED
                elif "cc*" in currency:
                    return UTILITIES.INCLUDED
                else:
                    return UTILITIES.UNKNOWN


class HousingPage(XMLPage):
    @method
    class get_housing(SeLogerItem):

        def obj_photos(self):
            photos = []

            for photo in XPath('./photos/photo')(self):
                url = CleanText('bigUrl', default=None)(photo)
                if not url:
                    url = CleanText('stdUrl', default=None)(photo)
                photos.append(HousingPhoto(url))
            return photos

        def obj_details(self):
            details = {}
            for detail in XPath('//detailAnnonce/details/detail')(self):
                details[CleanText('libelle')(detail)] = CleanText('valeur', default='N/A')(detail)

            details['Reference'] = CleanText('//detailAnnonce/reference')(self)
            return details

        obj_phone = CleanText('//contact/telephone')

        def obj_utilities(self):
            mention = CleanText('prixMention')(self)
            if "charges comprises" in mention:
                return UTILITIES.INCLUDED
            else:
                return UTILITIES.UNKNOWN



# -*- coding: utf-8 -*-

# Copyright(C) 2016 Vincent A
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

import os

from PyQt5.QtCore import Qt, QModelIndex, pyqtSlot as Slot
from PyQt5.QtGui import QCursor
from PyQt5.QtWidgets import QApplication

from weboob.tools.application.qt5 import QtMainWindow
from weboob.tools.application.qt5.backendcfg import BackendCfg
from weboob.tools.application.qt5.models import BackendListModel, ResultModel, FilterTypeModel
from weboob.tools.compat import range
from weboob.capabilities.collection import BaseCollection, CapCollection
from weboob.capabilities.gallery import CapGallery, BaseGallery
from weboob.capabilities.image import CapImage, BaseImage

from .ui.mainwindow_ui import Ui_MainWindow
from .viewer import Viewer


def size_format(n):
    UNITS = [
        (1 << 40, 'TiB'),
        (1 << 30, 'GiB'),
        (1 << 20, 'MiB'),
        (1 << 10, 'KiB'),
        (0, 'B')
    ]
    for f, u in UNITS:
        if n > f:
            return '%.2f %s' % (n / float(f), u)


class MainWindow(QtMainWindow):
    def __init__(self, config, storage, weboob, parent=None):
        super(MainWindow, self).__init__(parent)
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)

        self.mdl = ResultModel(weboob)
        self.mdl.setColumnFields([['name', 'title'],['url']])
        self.mdl.jobAdded.connect(self._jobAdded)
        self.mdl.jobFinished.connect(self._jobFinished)
        self.proxy_mdl = FilterTypeModel()
        self.proxy_mdl.setAcceptedTypes([BaseCollection, BaseGallery])
        self.proxy_mdl.setSourceModel(self.mdl)

        self.ui.collectionTree.setModel(self.proxy_mdl)
        self.ui.collectionTree.selectionModel().currentChanged.connect(self.showCollection)
        n = self.mdl.columnCount(QModelIndex())
        for i in range(n):
            self.ui.collectionTree.setColumnWidth(i, self.ui.collectionTree.width() // n)

        self.config = config
        self.storage = storage
        self.weboob = weboob

        self.ui.browseButton.clicked.connect(self.startBrowse)

        self.ui.searchGallEdit.returnPressed.connect(self.startGallSearch)
        self.ui.searchGallButton.clicked.connect(self.startGallSearch)

        self.ui.searchImgEdit.returnPressed.connect(self.startImgSearch)
        self.ui.searchImgButton.clicked.connect(self.startImgSearch)

        self.ui.imageList.setModel(self.mdl)
        self.ui.imageList.selectionModel().currentChanged.connect(self.showImageInfo)
        self.ui.imageList.activated.connect(self.openImage)

        self.fillBackends()

        if self.weboob.count_backends() == 0:
            self.backendsConfig()
        self.ui.actionBackends.triggered.connect(self.backendsConfig)

        self.ui.limitResults.valueChanged.connect(self._limitResultsChanged)
        self.mdl.setLimit(self.ui.limitResults.value())

        self.lastSaveDir = os.path.expanduser('~')

    @Slot()
    def backendsConfig(self):
        cfg = BackendCfg(self.weboob, (CapImage,), self)
        if cfg.run():
            self.fillBackends()

    def fillBackends(self):
        model = BackendListModel(self.weboob)
        model.addBackends(CapGallery)
        self.ui.backendGallCombo.setModel(model)

        model = BackendListModel(self.weboob)
        model.addBackends(CapImage)
        self.ui.backendImgCombo.setModel(model)

        model = BackendListModel(self.weboob)
        model.addBackends(CapImage, entry_title=True)
        model.addBackends(CapGallery, entry_title=True)
        self.ui.backendCollCombo.setModel(model)

    @Slot()
    def changeBackend(self):
        cap, backend = self.selectedBackend()

        if cap is CapImage:
            res_class = BaseImage
        else:
            res_class = BaseGallery
        self.mdl.setResourceClasses([res_class])

    @Slot(QModelIndex)
    def showCollection(self, qidx):
        qidx = self.proxy_mdl.mapToSource(qidx)
        qidx = qidx.sibling(qidx.row(), 0)
        self.ui.imageList.setRootIndex(qidx)

    @Slot(QModelIndex)
    def showGallery(self, qidx):
        self.ui.imageList.setEnabled(True)
        self.ui.imageList.show()
        qidx = qidx.sibling(qidx.row(), 0)
        self.ui.imageList.setRootIndex(qidx)

    def showNoneItem(self):
        self.ui.labelTitle.setText('-')
        self.ui.labelDescription.setText('-')
        self.ui.labelAuthor.setText('-')
        self.ui.labelDate.setText('-')
        self.ui.labelLink.setText('-')
        self.ui.labelSize.setText('-')
        self.ui.labelRating.setText('-')

    @Slot(QModelIndex)
    def showImageInfo(self, qidx):
        obj = qidx.data(ResultModel.RoleObject)
        if obj is None:
            self.showNoneItem()
            return

        if isinstance(obj, BaseImage):
            self.ui.labelTitle.setText(obj.title or '')
            self.ui.labelDescription.setText(obj.description or '')
            self.ui.labelAuthor.setText(obj.author or '')
            if obj.size:
                self.ui.labelSize.setText(size_format(obj.size) or '')
            else:
                self.ui.labelSize.setText('-')
            if obj.url:
                self.ui.labelLink.setText('<a href="%s">Link</a>' % obj.url)
            else:
                self.ui.labelLink.setText('')
        elif isinstance(obj, BaseGallery):
            pass

    @Slot(QModelIndex)
    def openImage(self, qidx):
        obj = qidx.data(ResultModel.RoleObject)

        if isinstance(obj, BaseImage):
            viewer = Viewer(self.weboob, self)
            viewer.jobAdded.connect(self._jobAdded)
            viewer.jobFinished.connect(self._jobFinished)
            viewer.setData(self.mdl, qidx)
            viewer.show()
        elif isinstance(obj, BaseGallery):
            self.ui.imageList.setRootIndex(qidx)

            qidx = self.proxy_mdl.mapFromSource(qidx)
            self.ui.collectionTree.setCurrentIndex(qidx)

    @Slot()
    def startImgSearch(self):
        backend = self.ui.backendImgCombo.currentData(BackendListModel.RoleBackendName)
        if not backend:
            backend = list(self.weboob.iter_backends(caps=CapImage))

        pattern = self.ui.searchImgEdit.text()
        if not pattern:
            return

        self.mdl.clear()

        self.ui.imageList.setRootIndex(QModelIndex())
        self.ui.collectionTree.setRootIndex(QModelIndex())
        self.mdl.addRootDoLimit(BaseImage, 'search_image', pattern, backends=backend)

    @Slot()
    def startGallSearch(self):
        backend = self.ui.backendGallCombo.currentData(BackendListModel.RoleBackendName)
        if not backend:
            backend = list(self.weboob.iter_backends(caps=CapGallery))

        pattern = self.ui.searchGallEdit.text()
        if not pattern:
            return

        self.mdl.clear()

        self.ui.imageList.setRootIndex(QModelIndex())
        self.ui.collectionTree.setRootIndex(QModelIndex())
        self.mdl.addRootDoLimit(BaseGallery, 'search_galleries', pattern, backends=backend)

    @Slot()
    def startBrowse(self):
        self.mdl.clear()

        cap = self.ui.backendCollCombo.currentData(BackendListModel.RoleCapability)
        backend = self.ui.backendCollCombo.currentData(BackendListModel.RoleBackendName)
        if backend:
            backends = [backend]
        else:
            backends = [b for b in self.weboob.iter_backends(caps=[cap]) if b.has_caps(CapCollection)]

        res_classes = [BaseImage, BaseGallery]
        self.mdl.setResourceClasses(res_classes)

        self.mdl.addRootDoLimit(None, 'iter_resources', res_classes, [], backends=backends)

    @Slot()
    def _jobAdded(self):
        QApplication.setOverrideCursor(QCursor(Qt.BusyCursor))

    @Slot()
    def _jobFinished(self):
        QApplication.restoreOverrideCursor()

    @Slot(int)
    def _limitResultsChanged(self, value):
        self.mdl.setLimit(value)



# -*- coding: utf-8 -*-

# Copyright(C) 2013 Julien Veyssier
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.
import re

from weboob.capabilities.base import UserError
from weboob.capabilities.calendar import CapCalendarEvent, CATEGORIES, BaseCalendarEvent
from weboob.capabilities.video import CapVideo, BaseVideo
from weboob.capabilities.collection import CapCollection, CollectionNotFound, Collection
from weboob.capabilities.cinema import CapCinema, Person, Movie
from weboob.tools.backend import Module

from .browser import AllocineBrowser

__all__ = ['AllocineModule']


class AllocineModule(Module, CapCinema, CapVideo, CapCalendarEvent, CapCollection):
    NAME = 'allocine'
    MAINTAINER = u'Julien Veyssier'
    EMAIL = 'julien.veyssier@aiur.fr'
    VERSION = '1.3'
    DESCRIPTION = u'AlloCiné French cinema database service'
    LICENSE = 'AGPLv3+'
    BROWSER = AllocineBrowser
    ASSOCIATED_CATEGORIES = [CATEGORIES.CINE]

    def get_movie(self, id):
        return self.browser.get_movie(id)

    def get_person(self, id):
        return self.browser.get_person(id)

    def iter_movies(self, pattern):
        return self.browser.iter_movies(pattern.encode('utf-8'))

    def iter_persons(self, pattern):
        return self.browser.iter_persons(pattern.encode('utf-8'))

    def iter_movie_persons(self, id, role=None):
        return self.browser.iter_movie_persons(id, role)

    def iter_person_movies(self, id, role=None):
        return self.browser.iter_person_movies(id, role)

    def iter_person_movies_ids(self, id):
        return self.browser.iter_person_movies_ids(id)

    def iter_movie_persons_ids(self, id):
        return self.browser.iter_movie_persons_ids(id)

    def get_person_biography(self, id):
        return self.browser.get_person_biography(id)

    def get_movie_releases(self, id, country=None):
        return self.browser.get_movie_releases(id, country)

    def fill_person(self, person, fields):
        if 'real_name' in fields or 'birth_place' in fields\
            or 'death_date' in fields or 'nationality' in fields\
            or 'short_biography' in fields or 'roles' in fields\
            or 'birth_date' in fields or 'thumbnail_url' in fields\
            or 'biography' in fields\
                or 'gender' in fields or fields is None:
            per = self.get_person(person.id)
            person.real_name = per.real_name
            person.birth_date = per.birth_date
            person.death_date = per.death_date
            person.birth_place = per.birth_place
            person.gender = per.gender
            person.nationality = per.nationality
            person.short_biography = per.short_biography
            person.short_description = per.short_description
            person.roles = per.roles
            person.biography = per.biography
            person.thumbnail_url = per.thumbnail_url

        return person

    def fill_movie(self, movie, fields):
        if 'other_titles' in fields or 'release_date' in fields\
            or 'duration' in fields or 'country' in fields\
            or 'roles' in fields or 'note' in fields\
                or 'thumbnail_url' in fields:
            mov = self.get_movie(movie.id)
            movie.other_titles = mov.other_titles
            movie.release_date = mov.release_date
            movie.duration = mov.duration
            movie.pitch = mov.pitch
            movie.country = mov.country
            movie.note = mov.note
            movie.roles = mov.roles
            movie.genres = mov.genres
            movie.short_description = mov.short_description
            movie.thumbnail_url = mov.thumbnail_url

        if 'all_release_dates' in fields:
            movie.all_release_dates = self.get_movie_releases(movie.id)

        return movie

    def fill_video(self, video, fields):
        if 'url' in fields:
            if not isinstance(video, BaseVideo):
                video = self.get_video(self, video.id)

            if hasattr(video, '_video_code'):
                video.url = unicode(self.browser.get_video_url(video._video_code))

        if 'thumbnail' in fields and video and video.thumbnail:
            video.thumbnail.data = self.browser.open(video.thumbnail.url)
        return video

    def get_video(self, _id):
        split_id = _id.split('#')
        if split_id[-1] == 'movie':
            return self.browser.get_movie_from_id(split_id[0])
        return self.browser.get_video_from_id(split_id[0], split_id[-1])

    def iter_resources(self, objs, split_path):
        if BaseVideo in objs:
            collection = self.get_collection(objs, split_path)
            if collection.path_level == 0:
                yield Collection([u'comingsoon'], u'Films prochainement au cinéma')
                yield Collection([u'nowshowing'], u'Films au cinéma')
                yield Collection([u'acshow'], u'Émissions')
                yield Collection([u'interview'], u'Interviews')
            if collection.path_level == 1:
                if collection.basename == u'acshow':
                    emissions = self.browser.get_emissions(collection.basename)
                    if emissions:
                        for emission in emissions:
                            yield emission
                elif collection.basename == u'interview':
                    videos = self.browser.get_categories_videos(collection.basename)
                    if videos:
                        for video in videos:
                            yield video
                else:
                    videos = self.browser.get_categories_movies(collection.basename)
                    if videos:
                        for video in videos:
                            yield video
            if collection.path_level == 2:
                videos = self.browser.get_categories_videos(':'.join(collection.split_path))
                if videos:
                    for video in videos:
                        yield video

    def validate_collection(self, objs, collection):
        if collection.path_level == 0:
            return
        if collection.path_level == 1 and (collection.basename in
                                           [u'comingsoon', u'nowshowing', u'acshow', u'interview']):
            return

        if collection.path_level == 2 and collection.parent_path == [u'acshow']:
            return

        raise CollectionNotFound(collection.split_path)

    def search_events(self, query):
        if CATEGORIES.CINE in query.categories:
            if query.city and re.match('\d{5}', query.city):
                events = list(self.browser.search_events(query))
                events.sort(key=lambda x: x.start_date, reverse=False)
                return events

            raise UserError('You must enter a zip code in city field')

    def get_event(self, id):
        return self.browser.get_event(id)

    def fill_event(self, event, fields):
        if 'description' in fields:
            movieCode = event.id.split('#')[2]
            movie = self.get_movie(movieCode)
            event.description = movie.pitch
        return event

    OBJECTS = {
        Person: fill_person,
        Movie: fill_movie,
        BaseVideo: fill_video,
        BaseCalendarEvent: fill_event
    }



# -*- coding: utf-8 -*-

# Copyright(C) 2013      Vincent A
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.tools.backend import Module, BackendConfig
from weboob.tools.value import Value, ValueBackendPassword
from weboob.capabilities.bugtracker import CapBugTracker, Issue, Project, User, Version, Status, Update, Attachment

from .browser import GithubBrowser


__all__ = ['GithubModule']


STATUSES = {'open': Status('open', u'open', Status.VALUE_NEW),
            'closed': Status('closed', u'closed', Status.VALUE_RESOLVED)}
# TODO tentatively parse github "labels"?


class GithubModule(Module, CapBugTracker):
    NAME = 'github'
    DESCRIPTION = u'GitHub issues tracking'
    MAINTAINER = u'Vincent A'
    EMAIL = 'dev@indigo.re'
    LICENSE = 'AGPLv3+'
    VERSION = '1.3'
    CONFIG = BackendConfig(Value('username', label='Username', default=''),
                           ValueBackendPassword('password', label='Password', default=''))

    BROWSER = GithubBrowser

    def create_default_browser(self):
        username = self.config['username'].get()
        if username:
            password = self.config['password'].get()
        else:
            password = None
        return self.create_browser(username, password)

    def get_project(self, _id):
        d = self.browser.get_project(_id)

        project = Project(_id, d['name'])
        project.members = list(self._iter_members(project.id))
        project.statuses = list(STATUSES.values())
        project.categories = []
        project.versions = list(self._iter_versions(project.id))

        return project

    def get_issue(self, _id):
        project_id, issue_number = self._extract_issue_id(_id)
        project = self.get_project(project_id)

        d = self.browser.get_issue(project_id, issue_number)

        issue = self._make_issue(d, project)
        if d['has_comments']:
            self._fetch_comments(issue)

        return issue

    def iter_issues(self, query):
        if ((query.assignee, query.author, query.status, query.title) ==
                                             (None, None, None, None)):
            it = self.browser.iter_project_issues(query.project)
        else:
            it = self.browser.iter_issues(query)

        project = self.get_project(query.project)
        for d in it:
            issue = self._make_issue(d, project)
            yield issue

    def create_issue(self, project_id):
        issue = Issue(0)
        issue.project = self.get_project(project_id)
        return issue

    def post_issue(self, issue):
        assert not issue.attachments
        if issue.id and issue.id != '0':
            _, issue_number = self._extract_issue_id(issue.id)
            self.browser.edit_issue(issue, issue_number)
        else:
            d = self.browser.post_issue(issue)
            issue.id = self._build_issue_id(issue.project.id, d['number'])
        return issue

    def update_issue(self, issue_id, update):
        try:
            issue_id = issue_id.id
        except AttributeError:
            pass

        assert not update.attachments
        self.browser.post_comment(issue_id, update.message)

    # iter_projects, remove_issue are impossible

    def _iter_members(self, project_id):
        for d in self.browser.iter_members(project_id):
            yield User(d['id'], d['name'])

    def _iter_versions(self, project_id):
        for d in self.browser.iter_milestones(project_id):
            yield Version(d['id'], d['name'])

    def _make_issue(self, d, project):
        _id = self._build_issue_id(project.id, d['number'])
        issue = Issue(_id)
        issue.url = d['url']
        issue.project = project
        issue.title = d['title']
        issue.body = d['body']
        issue.creation = d['creation']
        issue.updated = d['updated']
        issue.author = project.find_user(d['author'], None)
        if not issue.author:
            # may duplicate users
            issue.author = User(d['author'], d['author'])
        issue.status = STATUSES[d['status']]

        if d['assignee']:
            issue.assignee = project.find_user(d['assignee'], None)
        else:
            issue.assignee = None

        if d['version']:
            issue.version = project.find_version(d['version'], None)
        else:
            issue.version = None

        issue.category = None

        issue.attachments = [self._make_attachment(dattach) for dattach in d['attachments']]

        return issue

    def _fetch_comments(self, issue):
        project_id, issue_number = self._extract_issue_id(issue.id)
        if not issue.history:
            issue.history = []
        issue.history += [self._make_comment(dcomment, issue.project) for dcomment in self.browser.iter_comments(project_id, issue_number)]

    def _make_attachment(self, d):
        a = Attachment(d['url'])
        a.url = d['url']
        a.filename = d['filename']
        return a

    def _make_comment(self, d, project):
        u = Update(d['id'])
        u.message = d['message']
        u.author = project.find_user(d['author'], None)
        if not u.author:
            # may duplicate users
            u.author = User(d['author'], d['author'])
        u.date = d['date']
        u.changes = []
        u.attachments = [self._make_attachment(dattach) for dattach in d['attachments']]
        return u

    @staticmethod
    def _extract_issue_id(_id):
        return _id.rsplit('/', 1)

    @staticmethod
    def _build_issue_id(project_id, issue_number):
        return '%s/%s' % (project_id, issue_number)



"common tools for 20minutes backend"
# -*- coding: utf-8 -*-

# Copyright(C) 2011  Julien Hebert
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

import re


def url2id(url):
    "return an id from an url"
    regexp = re.compile("http://www.20min.fr/(\w+)/([0-9]+)/(.*$)")
    match = regexp.match(url)
    return '%s.%d.%s' % (match.group(1), int(match.group(2)), match.group(3))


def rssid(entry):
    return url2id(entry.id)



# -*- coding: utf-8 -*-

# Copyright(C) 2014      Bezleputh
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from weboob.browser import PagesBrowser, URL
from weboob.capabilities.housing import Query, TypeNotSupported
from .pages import CityListPage, HousingListPage, HousingPage, PhonePage


class LeboncoinBrowser(PagesBrowser):
    BASEURL = 'https://www.leboncoin.fr/'
    city = URL('ajax/location_list.html\?city=(?P<city>.*)&zipcode=(?P<zip>.*)', CityListPage)
    # Note: There is "ile_de_france" explicitly mentionned in the URL below.
    # This was a useful parameter in a previous version of Leboncoin, where
    # regions actually had some implications on the fetched posts. This is no
    # longer the case, and we can actually use any valid region to lookup post
    # in *any* region on Leboncoin. Then, we default to "ile_de_france".
    search = URL('(?P<type>.*)/offres/ile_de_france/occasions/\?(?P<_ps>ps|mrs)=(?P<ps>.*)&(?P<_pe>pe|mre)=(?P<pe>.*)&ros=(?P<ros>.*)&location=(?P<location>.*)&sqs=(?P<sqs>.*)&sqe=(?P<sqe>.*)&ret=(?P<ret>.*)&f=(?P<advert_type>.*)',
                 '(?P<_type>.*)/offres/ile_de_france/occasions.*?',
                 HousingListPage)
    housing = URL('ventes_immobilieres/(?P<_id>.*).htm', HousingPage)
    phone = URL('https://api.leboncoin.fr/api/utils/phonenumber.json', PhonePage)

    TYPES = {Query.TYPE_RENT: 'locations',
             Query.TYPE_SALE: 'ventes_immobilieres',
             Query.TYPE_SHARING: 'colocations', }

    RET = {Query.HOUSE_TYPES.HOUSE: '1',
           Query.HOUSE_TYPES.APART: '2',
           Query.HOUSE_TYPES.LAND: '3',
           Query.HOUSE_TYPES.PARKING: '4',
           Query.HOUSE_TYPES.OTHER: '5'}

    def __init__(self, *args, **kwargs):
        super(LeboncoinBrowser, self).__init__(*args, **kwargs)

    def get_cities(self, pattern):
        city = ''
        zip_code = ''
        if pattern.isdigit():
            zip_code = pattern
        else:
            city = pattern.replace(" ", "_")

        return self.city.go(city=city, zip=zip_code).get_cities()

    def search_housings(self, query, advert_type, module_name):
        if query.type not in self.TYPES:
            return TypeNotSupported()

        type, cities, nb_rooms, area_min, area_max, cost_min, cost_max, ret = self.decode_query(query, module_name)
        if len(cities) == 0 or len(ret) == 0:
            return list()

        return self.search.go(location=cities,
                              ros=nb_rooms,
                              sqs=area_min,
                              sqe=area_max,
                              _ps="mrs" if query.type == Query.TYPE_RENT else "ps",
                              ps=cost_min,
                              _pe="mre" if query.type == Query.TYPE_RENT else "pe",
                              pe=cost_max,
                              type=type,
                              advert_type=advert_type,
                              ret=ret).get_housing_list()

    def get_housing(self, _id, obj=None):
        housing = self.housing.go(_id=_id).get_housing(obj=obj)
        housing.phone = self.get_phone(_id)
        return housing

    def get_phone(self, _id):
        api_key = self.housing.stay_or_go(_id=_id).get_api_key()
        data = {'list_id': _id,
                'app_id': 'leboncoin_web_utils',
                'key': api_key,
                'text': 1, }
        return self.phone.go(data=data).get_phone()

    def decode_query(self, query, module_name):
        cities = [c.name for c in query.cities if c.backend == module_name]
        ret = [self.RET.get(g) for g in query.house_types if g in self.RET]
        _type = self.TYPES.get(query.type)

        self.search.go(_type=_type)

        nb_rooms = '' if not query.nb_rooms else self.page.get_rooms_min(query.nb_rooms)
        area_min = '' if not query.area_min else self.page.get_area_min(query.area_min)
        area_max = '' if not query.area_max else self.page.get_area_max(query.area_max)
        cost_min = '' if not query.cost_min else self.page.get_cost_min(query.cost_min, query.type)
        cost_max = '' if not query.cost_max else self.page.get_cost_max(query.cost_max, query.type)

        return _type, ','.join(cities), nb_rooms, area_min, area_max, cost_min, cost_max, '&ret='.join(ret)



# -*- coding: utf-8 -*-

# Copyright(C) 2015      Vincent A
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

from weboob.browser import LoginBrowser, URL, need_login
from weboob.browser.exceptions import HTTPNotFound
from weboob.exceptions import BrowserIncorrectPassword
from weboob.capabilities.image import Thumbnail

from .pages import PageLogin, PageDashboard, PageChapter, PageSection
from .video import MoocVideo

import re


class FunmoocBrowser(LoginBrowser):
    BASEURL = 'https://www.fun-mooc.fr'

    login = URL('/login', '/login_ajax', PageLogin)
    dashboard = URL('/dashboard', PageDashboard)
    course = URL('/courses/(?P<course>[^/]+/[^/]+/[^/]+)/courseware/?$',
                 '/courses/(?P<course>[^/]+/[^/]+/[^/]+)/info/?$',
                 PageChapter)
    chapter = URL('/courses/(?P<course>[^/]+/[^/]+/[^/]+)/courseware'
                  '/(?P<chapter>[0-9a-f]+)/$', PageChapter)
    section = URL('/courses/(?P<course>[^/]+/[^/]+/[^/]+)/courseware/'
                  '(?P<chapter>[0-9a-f]+)/(?P<section>[0-9a-f]+)/$', PageSection)

    file = URL(r'https://.*\.cloudfront\.net/videos/(?P<id>[^/]+)/'
               r'(?P<quality>\w+)\.mp4')

    def __init__(self, username, password, quality='hd', *args, **kwargs):
        super(FunmoocBrowser, self).__init__(username, password, *args, **kwargs)
        self.quality = quality

    def do_login(self):
        self.login.stay_or_go()
        csrf = self.session.cookies.get('csrftoken')
        self.page.login(self.username, self.password, csrf)
        self.dashboard.stay_or_go()
        if not self.page.logged:
            raise BrowserIncorrectPassword()

    def get_video(self, url):
        v = MoocVideo(url)
        v.url = url
        v.ext = 'mp4'
        v.title = re.sub(r'[:/"]', '-', url)
        return v

    @need_login
    def iter_videos(self, course, chapter, section):
        course = course.replace('-', '/')
        assert self.section.stay_or_go(course=course, chapter=chapter, section=section)

        for n, d in enumerate(self.page.iter_videos()):
            video = self.get_video(d['url'])
            if d.get('thumbnail'):
                video.thumbnail = Thumbnail(d['thumbnail'])
            if d.get('title'):
                video.title = d['title']
            yield video

    @need_login
    def iter_sections(self, courseid, chapter):
        course = courseid.replace('-', '/')
        assert self.chapter.stay_or_go(course=course, chapter=chapter)
        for coll in self.page.iter_sections():
            if coll.split_path[:2] == [courseid, chapter]:
                yield coll

    @need_login
    def iter_chapters(self, courseid):
        course = courseid.replace('-', '/')
        assert self.course.stay_or_go(course=course)
        return self.page.iter_chapters()

    @need_login
    def iter_courses(self):
        assert self.dashboard.stay_or_go()
        return self.page.iter_courses()

    @need_login
    def check_collection(self, path):
        if len(path) == 0:
            return True
        elif len(path) > 3:
            return False

        parts = list(zip(('course', 'chapter', 'section'), path))
        parts[0] = (parts[0][0], parts[0][1].replace('-', '/'))
        try:
            getattr(self, parts[-1][0]).open(**dict(parts))
        except HTTPNotFound:
            return False

        return True



# -*- coding: utf-8 -*-

# Copyright(C) 2013 Julien Veyssier
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from __future__ import print_function

import codecs

import requests
from PyQt5.QtCore import Qt, pyqtSlot as Slot
from PyQt5.QtGui import QImage, QPixmap
from PyQt5.QtWidgets import QFrame, QFileDialog

from weboob.applications.qcookboob.ui.recipe_ui import Ui_Recipe
from weboob.capabilities.base import empty


class Recipe(QFrame):
    def __init__(self, recipe, backend, parent=None):
        super(Recipe, self).__init__(parent)
        self.parent = parent
        self.ui = Ui_Recipe()
        self.ui.setupUi(self)

        self.ui.exportButton.clicked.connect(self.export)

        self.recipe = recipe
        self.backend = backend
        self.gotThumbnail()

        self.ui.idEdit.setText(u'%s@%s' % (recipe.id, backend.name))
        if not empty(recipe.title):
            self.ui.titleLabel.setText(recipe.title)
        if not empty(recipe.nb_person):
            nbstr = '-'.join(str(num) for num in recipe.nb_person)
            self.ui.nbPersonLabel.setText(nbstr)
        else:
            self.ui.nbPersonLabel.parent().hide()
        if not empty(recipe.preparation_time):
            self.ui.prepTimeLabel.setText('%s min' % recipe.preparation_time)
        else:
            self.ui.prepTimeLabel.parent().hide()
        if not empty(recipe.cooking_time):
            self.ui.cookingTimeLabel.setText('%s min' % recipe.cooking_time)
        else:
            self.ui.cookingTimeLabel.parent().hide()
        if not empty(recipe.ingredients):
            txt = u''
            for ing in recipe.ingredients:
                txt += u'* %s\n' % ing
            self.ui.ingredientsPlain.setPlainText(u'%s' % txt)
        else:
            self.ui.ingredientsPlain.parent().hide()
        if not empty(recipe.author):
            self.ui.authorLabel.setText(u'%s' % recipe.author)
        else:
            self.ui.authorLabel.parent().hide()
        if not empty(recipe.instructions):
            self.ui.instructionsPlain.setPlainText(u'%s' % recipe.instructions)
        else:
            self.ui.instructionsPlain.parent().hide()
        if not empty(recipe.comments):
            txt = u''
            for com in recipe.comments:
                txt += u'* %s\n' % com
            self.ui.commentsPlain.setPlainText(u'%s' % txt)
        else:
            self.ui.commentsPlain.parent().hide()

        self.ui.verticalLayout.setAlignment(Qt.AlignTop)
        self.ui.verticalLayout_2.setAlignment(Qt.AlignTop)

    def gotThumbnail(self):
        if self.recipe.picture_url and not empty(self.recipe.picture_url):
            data = requests.get(self.recipe.picture_url).content
            img = QImage.fromData(data)
            self.ui.imageLabel.setPixmap(QPixmap.fromImage(img).scaledToWidth(250, Qt.SmoothTransformation))

    @Slot()
    def export(self):
        fileDial = QFileDialog(self, u'Export "%s" recipe' %
                               self.recipe.title, u'%s.kreml' % self.recipe.id, 'Krecipe file (*.kreml);;all files (*)')
        fileDial.setAcceptMode(QFileDialog.AcceptSave)
        fileDial.setLabelText(QFileDialog.Accept, 'Export recipe')
        fileDial.setLabelText(QFileDialog.FileName, 'Recipe file name')
        ok = (fileDial.exec_() == 1)
        if not ok:
            return
        result = fileDial.selectedFiles()
        if len(result) > 0:
            dest = result[0]
            if not dest.endswith('.kreml'):
                dest += '.kreml'
            data = self.recipe.toKrecipesXml(author=self.backend.name)
            try:
                with codecs.open(dest, 'w', 'utf-8') as f:
                    f.write(data)
            except IOError as e:
                print(u'Unable to write Krecipe file in "%s": %s' % (dest, e), file=self.stderr)
                return 1
            return



# -*- coding: utf-8 -*-

# Copyright(C) 2010-2011 Romain Bignon
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from PyQt5.QtCore import pyqtSlot as Slot

from weboob.tools.application.qt5 import QtMainWindow
from weboob.tools.application.qt5.backendcfg import BackendCfg
from weboob.capabilities.messages import CapMessages

from .ui.main_window_ui import Ui_MainWindow
from .messages_manager import MessagesManager


class MainWindow(QtMainWindow):
    def __init__(self, config, weboob, parent=None):
        super(MainWindow, self).__init__(parent)
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)

        self.config = config
        self.weboob = weboob
        self.manager = MessagesManager(weboob, self)

        self.setCentralWidget(self.manager)

        self.ui.actionBackends.triggered.connect(self.backendsConfig)
        self.ui.actionRefresh.triggered.connect(self.refresh)

        if self.weboob.count_backends() == 0:
            self.backendsConfig()
        else:
            self.centralWidget().load()

    @Slot()
    def backendsConfig(self):
        bckndcfg = BackendCfg(self.weboob, (CapMessages,), self)
        if bckndcfg.run():
            self.centralWidget().load()

    @Slot()
    def refresh(self):
        self.centralWidget().refreshThreads()



# -*- coding: utf-8 -*-

# Copyright(C) 2013 Mathieu Jourdan
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

import StringIO
from weboob.deprecated.browser import Browser, BrowserIncorrectPassword
from .pages import LoginPage, HomePage, AccountPage, TimeoutPage, HistoryPage, PdfPage

__all__ = ['GdfSuez']


class GdfSuez(Browser):
    PROTOCOL = 'https'
    DOMAIN = 'www.gdfsuez-dolcevita.fr'
    PAGES = {'.*portail/clients.*?_nfpb=true&_pageLabel=page_identification':  LoginPage,
             '.*portail/clients.*?_nfpb=true&_pageLabel=page_accueil_compte_en_ligne': HomePage,
             '.*p/visualiser_mes_contrats.*?_nfpb=true': AccountPage,
             '.*p/page_historique_de_mes_factures': HistoryPage,
             '.*clients.*?_nfpb=true&_nfls=false&_pageLabel=page_erreur_timeout_session': TimeoutPage
             }

    loginp = '/portailClients/appmanager/portail/clients'
    homep = '/portailClients/appmanager/portail/clients?_nfpb=true&_pageLabel=page_accueil_compte_en_ligne'
    accountp = '/portailClients/client/p/visualiser_mes_contrats?_nfpb=true'
    historyp = '/portailClients/client/p/page_historique_de_mes_factures'

    def __init__(self, *args, **kwargs):
        Browser.__init__(self, *args, **kwargs)

    def home(self):
        self.location(self.homep)

    def is_logged(self):
        if self.is_on_page(LoginPage) or self.is_on_page(TimeoutPage):
            return False
        return True

    def login(self):
        assert isinstance(self.username, basestring)
        assert isinstance(self.password, basestring)
        #assert isemail(self.username)
        if not self.is_on_page(LoginPage):
            self.location(self.loginp)
        self.page.login(self.username, self.password)
        if self.is_on_page(LoginPage):
            raise BrowserIncorrectPassword()

    def get_subscription_list(self):
        if not self.is_on_page(AccountPage):
            self.location(self.accountp)
        return self.page.get_subscription_list()

    def get_subscription(self, id):
        assert isinstance(id, basestring)
        for sub in self.get_subscription_list():
            if sub.id == id:
                return sub

    def get_history(self, subscription):
        if not self.is_on_page(HistoryPage):
            self.location(self.historyp)
        return self.page.get_history()

    def get_details(self, subscription):
        bills = self.iter_documents()
        id = bills[0].id
        if not self.is_on_page(HistoryPage):
            self.location(self.historyp)
        url = 'https://www.gdfsuez-dolcevita.fr/' + self.get_document(id).url
        response = self.openurl(url)
        pdf = PdfPage(StringIO.StringIO(response.read()))
        for detail in pdf.get_details(subscription.label):
            yield detail

    def iter_documents(self):
        if not self.is_on_page(HistoryPage):
            self.location(self.historyp)
        return self.page.get_documents()

    def get_document(self, id):
        assert isinstance(id, basestring)
        for b in self.iter_documents():
            if b.id == id:
                return b



# -*- coding: utf-8 -*-

# Copyright(C) 2010-2011 Noé Rubinstein
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


import re

from weboob.capabilities.gallery import CapGallery, BaseGallery, BaseImage
from weboob.tools.backend import Module
from weboob.deprecated.browser import Browser, Page

__all__ = ['GenericComicReaderModule']


class DisplayPage(Page):
    def get_page(self, gallery):
        src = self.document.xpath(self.browser.params['img_src_xpath'])[0]

        return BaseImage(src,
                gallery=gallery,
                url=src)

    def page_list(self):
        return self.document.xpath(self.browser.params['page_list_xpath'])


class GenericComicReaderBrowser(Browser):
    def __init__(self, browser_params, *args, **kwargs):
        self.params = browser_params
        Browser.__init__(self, *args, **kwargs)

    def iter_gallery_images(self, gallery):
        self.location(gallery.url)
        assert self.is_on_page(DisplayPage)

        for p in self.page.page_list():
            if 'page_to_location' in self.params:
                self.location(self.params['page_to_location'] % p)
            else:
                self.location(p)

            assert self.is_on_page(DisplayPage)
            yield self.page.get_page(gallery)

    def fill_image(self, image, fields):
        if 'data' in fields:
            image.data = self.readurl(image.url)


class GenericComicReaderModule(Module, CapGallery):
    NAME = 'genericcomicreader'
    MAINTAINER = u'Noé Rubinstein'
    EMAIL = 'noe.rubinstein@gmail.com'
    VERSION = '1.3'
    DESCRIPTION = 'Generic comic reader backend; subclasses implement specific sites'
    LICENSE = 'AGPLv3+'
    BROWSER = GenericComicReaderBrowser

    BROWSER_PARAMS = {}
    ID_REGEXP = None
    URL_REGEXP = None
    ID_TO_URL = None
    PAGES = {}

    def create_default_browser(self):
        b = self.create_browser(self.BROWSER_PARAMS)
        b.PAGES = self.PAGES
        try:
            b.DOMAIN = self.DOMAIN
        except AttributeError:
            pass
        return b

    def iter_gallery_images(self, gallery):
        with self.browser:
            return self.browser.iter_gallery_images(gallery)

    def get_gallery(self, _id):
        match = re.match(r'^%s$' % self.URL_REGEXP, _id)
        if match:
            _id = match.group(1)
        else:
            match = re.match(r'^%s$' % self.ID_REGEXP, _id)
            if match:
                _id = match.group(0)
            else:
                return None

        gallery = BaseGallery(_id, url=(self.ID_TO_URL % _id))
        with self.browser:
            return gallery

    def fill_gallery(self, gallery, fields):
        gallery.title = gallery.id

    def fill_image(self, image, fields):
        with self.browser:
            self.browser.fill_image(image, fields)

    OBJECTS = {
            BaseGallery: fill_gallery,
            BaseImage: fill_image}



# -*- coding: utf-8 -*-

from weboob.capabilities.torrent import CapTorrent
from weboob.tools.backend import Module

from .browser import TorrentzBrowser


__all__ = ['TorrentzModule']


class TorrentzModule(Module, CapTorrent):
    NAME = 'torrentz'
    MAINTAINER = u'Matthieu Weber'
    EMAIL = 'weboob@weber.fi.eu.org'
    VERSION = '1.3'
    DESCRIPTION = 'Torrentz Search Engine.'
    LICENSE = 'AGPL'
    BROWSER = TorrentzBrowser

    def create_default_browser(self):
        return self.create_browser()

    def get_torrent(self, id):
        return self.browser.get_torrent(id)

    def get_torrent_file(self, id):
        return self.browser.get_torrent_file(id)

    def iter_torrents(self, pattern):
        return self.browser.iter_torrents(pattern.replace(' ', '+'))



# -*- coding: utf-8 -*-

# Copyright(C) 2016 Julien Veyssier
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.capabilities.lyrics import SongLyrics
from weboob.capabilities.base import NotLoaded, NotAvailable

from weboob.browser.elements import ItemElement, ListElement, method
from weboob.browser.pages import HTMLPage
from weboob.browser.filters.standard import Regexp, CleanText, Format
from weboob.browser.filters.html import CleanHTML

import random


class HomePage(HTMLPage):
    def search_lyrics(self, criteria, pattern):
        form = self.get_form(xpath='//form[@name="rechercher"]')
        form['query'] = pattern
        if criteria == 'artist':
            form['termes_a'] = pattern
        else:
            form['termes_t'] = pattern
        form.submit()


class ArtistResultsPage(HTMLPage):
    def get_artist_ids(self):
        artists_href = self.doc.xpath('//div[has-class("cont_cat")]//a[has-class("matchA")]/@href')
        aids = [href.split('/')[-1].replace('paroles-','') for href in artists_href]
        return aids


class ArtistSongsPage(HTMLPage):
    @method
    class iter_lyrics(ListElement):
        item_xpath = '//td[has-class("art_titr")]//a'

        class item(ItemElement):
            klass = SongLyrics

            obj_title = CleanText('.', default=NotAvailable)
            obj_artist = CleanText('//h1[@id="art_title"]', default=NotAvailable)
            # little trick because the damn site potentially shows identical songs in results
            # the dummy added prefix number does not annoy the module
            # it seems this part of the URL is not red anyway
            def obj_id(self):
                res = Format('%s%s',
                        int(random.random()*100000),
                        Regexp(CleanText('./@href', default=NotAvailable), 'paroles-(.*)'))(self)
                return res
            obj_content = NotLoaded


class SongResultsPage(HTMLPage):
    @method
    class iter_lyrics(ListElement):
        item_xpath = '//div[has-class("cont_cat")]//table//tr[position() > 1]'

        class item(ItemElement):
            klass = SongLyrics

            obj_title = CleanText('.//a[has-class("matchT")]', default=NotAvailable)
            obj_id = Regexp(CleanText('.//a[has-class("matchT")]/@href', default=NotAvailable), 'paroles-(.*)')
            obj_artist = CleanText('.//a[has-class("matchA")]', default=NotAvailable)
            obj_content = NotLoaded


class SonglyricsPage(HTMLPage):
    @method
    class get_lyrics(ItemElement):
        klass = SongLyrics

        obj_content = CleanText(CleanHTML('//div[@id="lyr_scroll"]', default=NotAvailable), newlines=False)
        obj_title = CleanText('//div[@id="main_ct"]//ul[has-class("semiopaquemenu")]//li[position()=3]', default=NotAvailable)
        obj_artist = CleanText('//div[@id="main_ct"]//ul[has-class("semiopaquemenu")]//li[position()=2]', default=NotAvailable)
        obj_id = Regexp(CleanText('//div[@id="main_ct"]//ul[has-class("semiopaquemenu")]//li[position()=3]//a/@href', default=NotAvailable),
                 'paroles-(.*)')




# -*- coding: utf-8 -*-

# Copyright(C) 2013      Romain Bignon
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from collections import OrderedDict

from weboob.tools.backend import Module, BackendConfig
from weboob.tools.value import Value
from weboob.capabilities.travel import CapTravel, Station, Departure
from weboob.capabilities import UserError

from .browser import VoyagesSNCFBrowser


__all__ = ['VoyagesSNCFModule']


class VoyagesSNCFModule(Module, CapTravel):
    NAME = 'voyagessncf'
    DESCRIPTION = u'Voyages SNCF'
    MAINTAINER = u'Romain Bignon'
    EMAIL = 'romain@weboob.org'
    LICENSE = 'AGPLv3+'
    VERSION = '1.3'
    CONFIG = BackendConfig(Value('age', label='Passenger age', default='ADULT',
                                 choices=OrderedDict((('ADULT', '26-59 ans'),
                                                      ('SENIOR', '60 et +'),
                                                      ('YOUNG', '12-25 ans'),
                                                      ('CHILD_UNDER_FOUR', '0-3 ans'),
                                                      ('CHILDREN', '4-11 ans')))),
                           Value('card', label='Passenger card', default='default',
                                 choices=OrderedDict((('default', u'Pas de carte'),
                                                      ('YOUNGS', u'Carte Jeune'),
                                                      ('ESCA', u'Carte Escapades'),
                                                      ('WEEKE', u'Carte Week-end'),
                                                      ('FQ2ND', u'Abo Fréquence 2e'),
                                                      ('FQ1ST', u'Abo Fréquence 1e'),
                                                      ('FF2ND', u'Abo Forfait 2e'),
                                                      ('FF1ST', u'Abo Forfait 1e'),
                                                      ('ACCWE', u'Accompagnant Carte Week-end'),
                                                      ('ACCCHD', u'Accompagnant Carte Enfant+'),
                                                      ('ENFAM', u'Carte Enfant Famille'),
                                                      ('FAM30', u'Carte Familles Nombreuses 30%'),
                                                      ('FAM40', u'Carte Familles Nombreuses 40%'),
                                                      ('FAM50', u'Carte Familles Nombreuses 50%'),
                                                      ('FAM75', u'Carte Familles Nombreuses 75%'),
                                                      ('MI2ND', u'Carte Militaire 2e'),
                                                      ('MI1ST', u'Carte Militaire 1e'),
                                                      ('MIFAM', u'Carte Famille Militaire'),
                                                      ('THBIZ', u'Thalys ThePass Business'),
                                                      ('THPREM', u'Thalys ThePass Premium'),
                                                      ('THWE', u'Thalys ThePass Weekend')))),
                           Value('class', label='Comfort class', default='2',
                                 choices=OrderedDict((('1', u'1e classe'),
                                                      ('2', u'2e classe')))))

    BROWSER = VoyagesSNCFBrowser
    STATIONS = []

    def _populate_stations(self):
        if len(self.STATIONS) == 0:
            with self.browser:
                self.STATIONS = self.browser.get_stations()

    def iter_station_search(self, pattern):
        self._populate_stations()

        pattern = pattern.lower()
        already = set()

        # First stations whose name starts with pattern...
        for _id, name in enumerate(self.STATIONS):
            if name.lower().startswith(pattern):
                already.add(_id)
                yield Station(_id, unicode(name))
        # ...then ones whose name contains pattern.
        for _id, name in enumerate(self.STATIONS):
            if pattern in name.lower() and _id not in already:
                yield Station(_id, unicode(name))

    def iter_station_departures(self, station_id, arrival_id=None, date=None):
        self._populate_stations()

        if arrival_id is None:
            raise UserError('The arrival station is required')

        try:
            station = self.STATIONS[int(station_id)]
            arrival = self.STATIONS[int(arrival_id)]
        except (IndexError, ValueError):
            try:
                station = list(self.iter_station_search(station_id))[0].name
                arrival = list(self.iter_station_search(arrival_id))[0].name
            except IndexError:
                raise UserError('Unknown station')

        with self.browser:
            for i, d in enumerate(self.browser.iter_departures(station, arrival, date,
                                                               self.config['age'].get(),
                                                               self.config['card'].get(),
                                                               self.config['class'].get())):
                departure = Departure(i, d['type'], d['time'])
                departure.departure_station = d['departure']
                departure.arrival_station = d['arrival']
                departure.arrival_time = d['arrival_time']
                departure.price = d['price']
                departure.currency = d['currency']
                departure.information = d['price_info']
                yield departure



# -*- coding: utf-8 -*-

# Copyright(C) 2013-2014  Florent Fourcot
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

import re
from decimal import Decimal

from weboob.capabilities.base import NotAvailable
from weboob.capabilities.bank import Investment
from weboob.browser.pages import RawPage, HTMLPage, LoggedPage, pagination
from weboob.browser.elements import ListElement, TableElement, ItemElement, method
from weboob.browser.filters.standard import CleanDecimal, CleanText, Date, TableCell, Regexp, Env, Async, AsyncLoad
from weboob.browser.filters.html import Link, Attr
from weboob.tools.capabilities.bank.transactions import FrenchTransaction
from weboob.tools.compat import unicode

class NetissimaPage(HTMLPage):
    pass

class Transaction(FrenchTransaction):
    pass


class TitreValuePage(LoggedPage, HTMLPage):
    def get_isin(self):
        return unicode(self.doc.xpath('//div[@id="headFiche"]//span[@id="test3"]/text()')[0].split(' - ')[0].strip())


class TitrePage(LoggedPage, RawPage):
    def build_doc(self, content):
        return content.decode(self.encoding)

    def get_balance(self):
        return CleanDecimal(default=None, replace_dots=True).filter(self.doc.split('{')[0])

    def iter_investments(self, account):
        # We did not get some html, but something like that (XX is a quantity, YY a price):
        # "message='<total> &euro;{<total> &euro;{0,01 &euro;{<liquidity> &euro;{0,00{{05/17{{03/05/2017{11:06{-XX &euro;{710TI81000029397EUR{XX &euro;{XX &euro;{|OPHTHOTECH(NASDAQ)#cotationValeur.php?val=OPHT&amp;pl=11&amp;nc=2&amp;
        # popup=2{6{E:ALO{PAR{{reel{695{380{ALSTOM REGROUPT#XX#YY,YY &euro;#YY,YY &euro;#1 YYY,YY &euro;#-YYY,YY &euro;#-42,42%#-0,98 %#42,42 %#|1|AXA#cotationValeur.php?val=E:CS&amp;pl=6&amp;nc=1&amp;
        # popup=2{6{E:CS{PAR{{reel{695{380{AXA#XX#YY,YY &euro;#YY,YYY &euro;#YYY,YY &euro;#YY,YY &euro;#3,70%#42,42 %#42,42 %#|1|blablablab #cotationValeur.php?val=P:CODE&amp;pl=6&amp;nc=1&amp;
        # [...]
        data = self.browser.cache["investments_data"].get(account.id, self.doc)
        lines = data.split("|1|")
        message = lines[0]
        if len(lines) > 1:
            start = 1
            lines[0] = lines[0].split("|")[1]
        else:
            start = 0
            lines = data.split("popup=2")
            lines.pop(0)
        invests = []
        for line in lines:
            _id, _pl = None, None
            columns = line.split('#')
            if columns[1] != '':
                _pl = columns[start].split('{')[1]
                _id = columns[start].split('{')[2]
            invest = Investment()
            invest.label = columns[start].split('{')[-1]
            invest.code = _id or NotAvailable
            if invest.code and ':' in invest.code:
                invest.code = self.browser.titrevalue.open(val=invest.code,pl=_pl).get_isin()
            # The code we got is not a real ISIN code.
            if invest.code and not re.match('^[A-Z]{2}[\d]{10}$|^[A-Z]{2}[\d]{5}[A-Z]{1}[\d]{4}$', invest.code):
                m = re.search('\{([A-Z]{2}[\d]{10})\{|\{([A-Z]{2}[\d]{5}[A-Z]{1}[\d]{4})\{', line)
                if m:
                    invest.code = m.group(1) or m.group(2)

            for x, attr in enumerate(['quantity', 'unitprice', 'unitvalue', 'valuation', 'diff'], 1):
                currency = FrenchTransaction.Currency().filter(columns[start + x])
                amount = CleanDecimal(default=NotAvailable).filter(FrenchTransaction.clean_amount(columns[start + x]))
                if currency and currency != account.currency:
                    invest.original_currency = currency
                    attr = "original_" + attr
                setattr(invest, attr, amount)
            # valuation is not nullable, use 0 as default value
            if not invest.valuation:
                invest.valuation = Decimal('0')

            # On some case we have a multine investment with a total column
            # for now we have only see this on 2 lines, we will need to adapt it when o
            if columns[9 if start == 0 else 0] == u'|Total' and _id == 'fichevaleur':
                prev_inv = invest
                invest = invests.pop(-1)
                if prev_inv.quantity:
                    invest.quantity = invest.quantity + prev_inv.quantity
                if prev_inv.valuation:
                    invest.valuation = invest.valuation + prev_inv.valuation
                if prev_inv.diff:
                    invest.diff = invest.diff + prev_inv.diff

            invests.append(invest)

        # There is no investment on life insurance in the process to be created.
        if len(message.split('&')) >= 4:
            # We also have to get the liquidity as an investment.
            invest = Investment()
            invest.label = "Liquidités"
            invest.code = "XX-liquidity"
            invest.valuation = CleanDecimal(None, True).filter(message.split('&')[3].replace('euro;{','').strip())
            invests.append(invest)
        for invest in invests:
            yield invest


class TitreHistory(LoggedPage, HTMLPage):
    @method
    class iter_history(ListElement):
        item_xpath = '//table[@class="datas retour"]/tr'

        class item(ItemElement):
            klass = Transaction

            condition = lambda self: len(self.el.xpath('td[@class="impaire"]')) > 0

            obj_raw = Transaction.Raw('td[4] | td[3]')
            obj_date = Date(CleanText('td[2]'), dayfirst=True)
            obj_amount = CleanDecimal('td[7]', replace_dots=True)


class ASVHistory(LoggedPage, HTMLPage):
    @method
    class get_investments(TableElement):
        item_xpath = '//table[@class="Tableau"]/tr[td[not(has-class("enteteTableau"))]]'
        head_xpath = '//table[@class="Tableau"]/tr[td[has-class("enteteTableau")]]/td'

        col_label = u'Support(s)'
        col_vdate = u'Date de valeur'
        col_unitvalue = u'Valeur de part'
        col_quantity = [u'(*) Nb de parts', u'Nb de parts']
        col_valuation = [u'Montant', u'Montant versé']

        class item(ItemElement):
            klass = Investment

            load_details = Regexp(Attr('./td/a', 'onclick', default=""), 'PageExterne\(\'([^\']+)', default=None) & AsyncLoad

            obj_label = CleanText(TableCell('label'))
            obj_code = Async('details') & CleanText('//td[contains(text(), "CodeISIN")]/b', default=NotAvailable)
            obj_quantity = CleanDecimal(TableCell('quantity'), replace_dots=True, default=NotAvailable)
            obj_unitvalue = CleanDecimal(TableCell('unitvalue'), replace_dots=True, default=NotAvailable)
            obj_valuation = CleanDecimal(TableCell('valuation'), replace_dots=True)
            obj_vdate = Date(CleanText(TableCell('vdate')), dayfirst=True)

    @pagination
    @method
    class iter_history(TableElement):
        item_xpath = '//table[@class="Tableau"]/tr[td[not(has-class("enteteTableau"))]]'
        head_xpath = '//table[@class="Tableau"]/tr[td[has-class("enteteTableau")]]/td'

        col_date = u'Date d\'effet'
        col_raw = u'Nature du mouvement'
        col_amount = u'Montant brut'

        next_page = Link('//a[contains(@href, "PageSuivante")]', default=None)

        class item(ItemElement):
            klass = Transaction

            obj_date = Date(CleanText(TableCell('date')), dayfirst=True)
            obj_raw = Transaction.Raw(TableCell('raw'))
            obj_amount = CleanDecimal(TableCell('amount'), replace_dots=True)
            obj__detail = Env('detail')

            def obj_id(self):
                try:
                    return Regexp(Link('./td/a', default=None), 'numMvt=(\d+)', default=None)(self)
                except TypeError:
                    return NotAvailable

            def parse(self, el):
                link = Link('./td/a', default=None)(self)
                page = self.page.browser.async_open(link) if link else None
                self.env['detail'] = page



# -*- coding: utf-8 -*-

# Copyright(C) 2010-2011  Romain Bignon
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.tools.backend import Module, BackendConfig
from weboob.tools.newsfeed import Newsfeed
from weboob.tools.value import Value, ValueInt, ValueBackendPassword
from weboob.tools.misc import limit
from weboob.capabilities.messages import CapMessages, CapMessagesPost, Message, Thread, CantSendMessage

from .browser import PhpBB
from .tools import rssid, url2id, id2url, id2topic


__all__ = ['PhpBBModule']


class PhpBBModule(Module, CapMessages, CapMessagesPost):
    NAME = 'phpbb'
    MAINTAINER = u'Romain Bignon'
    EMAIL = 'romain@weboob.org'
    VERSION = '1.3'
    LICENSE = 'AGPLv3+'
    DESCRIPTION = "phpBB forum"
    CONFIG = BackendConfig(Value('url',                     label='URL of forum', regexp='https?://.*'),
                           Value('username',                label='Username', default=''),
                           ValueBackendPassword('password', label='Password', default=''),
                           ValueInt('thread_unread_messages', label='Limit number of unread messages to retrieve for a thread', default=500)
                          )
    STORAGE = {'seen': {}}
    BROWSER = PhpBB

    def create_default_browser(self):
        username = self.config['username'].get()
        if len(username) > 0:
            password = self.config['password'].get()
        else:
            password = None
        return self.create_browser(self.config['url'].get(),
                                   username, password)

    #### CapMessages ##############################################

    def _iter_threads(self, root_link=None):
        links = list(self.browser.iter_links(root_link.url if root_link else None))

        for link in links:
            if link.type == link.FORUM:
                link.title = '%s[%s]' % (root_link.title if root_link else '', link.title)
                for thread in self._iter_threads(link):
                    yield thread
            if link.type == link.TOPIC:
                thread = Thread(url2id(link.url))
                thread.title = ('%s ' % root_link.title if root_link else '') + link.title
                thread.date = link.date
                thread.flags = thread.IS_DISCUSSION
                yield thread

    def iter_threads(self):
        return self._iter_threads()

    def get_thread(self, id):
        thread = None
        parent = None

        if isinstance(id, Thread):
            thread = id
            id = thread.id

        thread_id = url2id(id, nopost=True) or id
        try:
            last_seen_id = self.storage.get('seen', default={})[id2topic(thread_id)]
        except KeyError:
            last_seen_id = 0

        for post in self.browser.iter_posts(id):
            if not thread:
                thread = Thread(thread_id)
                thread.title = post.title

            m = self._post2message(thread, post)
            m.parent = parent
            if last_seen_id < post.id:
                m.flags |= Message.IS_UNREAD

            if parent:
                parent.children = [m]
            else:
                thread.root = m

            parent = m

        return thread

    def _post2message(self, thread, post):
        signature = post.signature
        if signature:
            signature += '<br />'
        signature += 'URL: %s' % self.browser.absurl(id2url('%s.%s' % (thread.id, post.id)))
        return Message(thread=thread,
                       id=post.id,
                       title=post.title,
                       sender=post.author,
                       receivers=None,
                       date=post.date,
                       parent=None,
                       content=post.content,
                       signature=signature,
                       children=[],
                       flags=Message.IS_HTML)

    def iter_unread_messages(self):
        url = self.browser.get_root_feed_url()
        for article in Newsfeed(url, rssid).iter_entries():
            id = url2id(article.link)
            thread = None

            try:
                last_seen_id = self.storage.get('seen', default={})[id2topic(id)]
            except KeyError:
                last_seen_id = 0

            child = None
            iterator = self.browser.riter_posts(id, last_seen_id)
            if self.config['thread_unread_messages'].get() > 0:
                iterator = limit(iterator, self.config['thread_unread_messages'].get())
            for post in iterator:
                if not thread:
                    thread = Thread('%s.%s' % (post.forum_id, post.topic_id))
                message = self._post2message(thread, post)

                if child:
                    message.children.append(child)
                    child.parent = message

                if post.parent:
                    message.parent = Message(thread=thread,
                                             id=post.parent)
                else:
                    thread.root = message
                yield message

    def set_message_read(self, message):
        try:
            last_seen_id = self.storage.get('seen', default={})[id2topic(message.thread.id)]
        except KeyError:
            last_seen_id = 0

        if message.id > last_seen_id:
            self.storage.set('seen', id2topic(message.thread.id), message.id)
            self.storage.save()

    def fill_thread(self, thread, fields):
        return self.get_thread(thread)

    #### CapMessagesReply #########################################
    def post_message(self, message):
        assert message.thread

        forum = 0
        topic = 0
        if message.thread:
            try:
                if '.' in message.thread.id:
                    forum, topic = [int(i) for i in message.thread.id.split('.', 1)]
                else:
                    forum = int(message.thread.id)
            except ValueError:
                raise CantSendMessage('Thread ID must be in form "FORUM_ID[.TOPIC_ID]".')

        return self.browser.post_answer(forum,
                                        topic,
                                        message.title,
                                        message.content)

    OBJECTS = {Thread: fill_thread}



# -*- coding: utf-8 -*-

# Copyright(C) 2013      Bezleputh
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from collections import OrderedDict

from weboob.capabilities.job import BaseJobAdvert
from weboob.tools.backend import Module, BackendConfig
from weboob.capabilities.job import CapJob
from weboob.tools.value import Value
from .browser import ApecBrowser
from .job import APEC_CONTRATS, APEC_EXPERIENCE

__all__ = ['ApecModule']


class ApecModule(Module, CapJob):
    NAME = 'apec'
    DESCRIPTION = u'apec website'
    MAINTAINER = u'Bezleputh'
    EMAIL = 'carton_ben@yahoo.fr'
    VERSION = '1.3'

    BROWSER = ApecBrowser

    places_choices = OrderedDict([(k, u'%s' % (v)) for k, v in sorted({
        '001|99700': u'UE Hors France',
        '002|99126': u'..Grèce',
        '003|99132': u'..Royaume Uni',
        '004|99134': u'..Espagne',
        '005|99136': u'..Irlande',
        '006|99139': u'..Portugal',
        '007|99254': u'..Chypre',
        '008|99127': u'..Italie',
        '009|99131': u'..Belgique',
        '010|99135': u'..Pays Bas',
        '011|99137': u'..Luxembourg',
        '012|99144': u'..Malte',
        '013|99145': u'..Slovénie',
        '014|99101': u'..Danemark',
        '015|99104': u'..Suède',
        '016|99105': u'..Finlande',
        '017|99106': u'..Estonie',
        '018|99107': u'..Lettonie',
        '019|99108': u'..Lituanie',
        '020|99109': u'..Allemagne',
        '021|99110': u'..Autriche',
        '022|99111': u'..Bulgarie',
        '023|99112': u'..Hongrie',
        '024|99114': u'..Roumanie',
        '025|99116': u'..République Tchèque',
        '026|99117': u'..Slovaquie',
        '027|99119': u'..Croatie',
        '028|99122': u'..Pologne',
        '029|799': u'France',
        '030|711': u'..Ile-de-France',
        '031|75': u'....Paris',
        '032|77': u'....Seine-et-Marne',
        '033|78': u'....Yvelines',
        '034|91': u'....Essonne',
        '035|92': u'....Hauts-de-Seine',
        '036|93': u'....Seine-Saint-Denis',
        '037|94': u'....Val-de-Marne',
        '038|95': u'....Val-d\'Oise',
        '039|703': u'..Basse-Normandie',
        '040|14': u'....Calvados',
        '041|50': u'....Manche',
        '042|61': u'....Orne',
        '043|705': u'..Bretagne',
        '044|22': u'....Côtes d\'Armor',
        '045|29': u'....Finistère',
        '046|35': u'....Ille-et-Vilaine',
        '047|56': u'....Morbihan',
        '048|706': u'..Centre',
        '049|18': u'....Cher',
        '050|28': u'....Eure-et-Loir',
        '051|36': u'....Indre',
        '052|37': u'....Indre-et-Loire',
        '053|41': u'....Loir-et-Cher',
        '054|45': u'....Loiret',
        '055|710': u'..Haute-Normandie',
        '056|27': u'....Eure',
        '057|76': u'....Seine-Maritime',
        '058|717': u'..Pays de La Loire',
        '059|44': u'....Loire-Atlantique',
        '060|49': u'....Maine-et-Loire',
        '061|53': u'....Mayenne',
        '062|72': u'....Sarthe',
        '063|85': u'....Vendée',
        '064|700': u'..Alsace',
        '065|67': u'....Bas-Rhin',
        '066|68': u'....Haut-Rhin',
        '067|704': u'..Bourgogne',
        '068|21': u'....Côte d\'Or',
        '069|58': u'....Nièvre',
        '070|71': u'....Saône-et-Loire',
        '071|89': u'....Yonne',
        '072|707': u'..Champagne',
        '073|8': u'....Ardennes',
        '074|10': u'....Aube',
        '075|51': u'....Marne',
        '076|52': u'....Haute-Marne',
        '077|709': u'..Franche-Comté',
        '078|25': u'....Doubs',
        '079|39': u'....Jura',
        '080|70': u'....Haute-Saône',
        '081|90': u'....Territoire de Belfort',
        '082|714': u'..Lorraine',
        '083|54': u'....Meurthe-et-Moselle',
        '084|55': u'....Meuse',
        '085|57': u'....Moselle',
        '086|88': u'....Vosges',
        '087|716': u'..Nord-Pas-de-Calais',
        '088|59': u'....Nord',
        '089|62': u'....Pas-de-Calais',
        '090|718': u'..Picardie',
        '091|2': u'....Aisne',
        '092|60': u'....Oise',
        '093|80': u'....Somme',
        '094|20': u'..Corse',
        '095|750': u'....Corse du Sud',
        '096|751': u'....Haute-Corse',
        '097|702': u'..Auvergne',
        '098|3': u'....Allier',
        '099|15': u'....Cantal',
        '100|43': u'....Haute-Loire',
        '101|63': u'....Puy-de-Dôme',
        '102|720': u'..PACA',
        '103|4': u'....Alpes-de-Haute-Provence',
        '104|5': u'....Hautes-Alpes',
        '105|6': u'....Alpes-Maritimes',
        '106|13': u'....Bouches-du-Rhône',
        '107|83': u'....Var',
        '108|84': u'....Vaucluse',
        '109|721': u'..Rhône-Alpes',
        '110|1': u'....Ain',
        '111|7': u'....Ardèche',
        '112|26': u'....Drôme',
        '113|38': u'....Isère',
        '114|42': u'....Loire',
        '115|69': u'....Rhône',
        '116|73': u'....Savoie',
        '117|74': u'....Haute-Savoie',
        '118|701': u'..Aquitaine',
        '119|24': u'....Dordogne',
        '120|33': u'....Gironde',
        '121|40': u'....Landes',
        '122|47': u'....Lot-et-Garonne',
        '123|64': u'....Pyrénées-Atlantiques',
        '124|712': u'..Languedoc-Roussillon',
        '125|11': u'....Aude',
        '126|30': u'....Gard',
        '127|34': u'....Hérault',
        '128|48': u'....Lozère',
        '129|66': u'....Pyrénées-Orientales',
        '130|713': u'..Limousin',
        '131|19': u'....Corrèze',
        '132|23': u'....Creuse',
        '133|87': u'....Haute-Vienne',
        '134|715': u'..Midi-Pyrénées',
        '135|9': u'....Ariège',
        '136|12': u'....Aveyron',
        '137|31': u'....Haute-Garonne',
        '138|32': u'....Gers',
        '139|46': u'....Lot',
        '140|65': u'....Hautes-Pyrénées',
        '141|81': u'....Tarn',
        '142|82': u'....Tarn-et-Garonne',
        '143|719': u'..Poitou-Charentes',
        '144|16': u'....Charente',
        '145|17': u'....Charente-Maritime',
        '146|79': u'....Deux-Sèvres',
        '147|86': u'....Vienne',
        '148|99712': u'..France Outre-Mer',
        '149|99519': u'....Terres Australes et Antarctiques Françaises',
        '150|97100': u'....Guadeloupe',
        '151|97200': u'....Martinique',
        '152|97300': u'....Guyane',
        '153|97400': u'....La Réunion',
        '154|97500': u'....Saint-Pierre-et-Miquelon',
        '155|97600': u'....Mayotte',
        '156|98300': u'....Polynésie Française',
        '157|98600': u'....Wallis et Futuna',
        '158|98800': u'....Nouvelle Calédonie',
        '159|97800': u'....Saint-Martin',
        '160|97700': u'....Saint-Barthélémy',
        '161|102099': u'International',
        '162|99715': u'..Afrique',
        '163|99716': u'..Asie',
        '164|99700': u'..UE Hors France',
        '165|99701': u'..Europe Hors UE',
        '166|99702': u'..Amérique du Nord',
        '167|99711': u'..Océanie',
        '168|99714': u'..Amérique Latine',
    }.items())])

    fonction_choices = OrderedDict([(k, u'%s' % (v)) for k, v in sorted({
        '00|': u'-- Indifférent --',
        '01|101828': u'Commercial, Marketing',
        '02|101782': u'.....Administration des ventes et SAV',
        '03|101783': u'.....Chargé d\'affaires, technico-commercial',
        '04|101784': u'.....Commercial',
        '05|101785': u'.....Commerce international',
        '06|101786': u'.....Direction commerciale et marketing',
        '07|101787': u'.....Direction régionale et d\'agence',
        '08|101788': u'.....Marketing',
        '09|101789': u'.....Ventes en magasin',
        '10|101829': u'Communication, Création',
        '11|101790': u'.....Communication',
        '12|101791': u'.....Création',
        '13|101792': u'.....Documentation, rédaction technique',
        '14|101793': u'.....Journalisme, édition',
        '15|101830': u'Direction d\'entreprise',
        '16|101794': u'.....Adjoint, conseil de direction',
        '17|101795': u'.....Direction générale',
        '18|101831': u'Etudes, Recherche et Développement',
        '19|101796': u'.....Conception, recherche',
        '20|101797': u'.....Direction recherche et développement',
        '21|101798': u'.....Etudes socio-économiques',
        '22|101799': u'.....Projets scientifiques et techniques',
        '23|101800': u'.....Test, essai, validation, expertise',
        '24|101832': u'Gestion, Finance, Administration',
        '25|101801': u'.....Administration, gestion, organisation',
        '26|101802': u'.....Comptabilité',
        '27|101803': u'.....Contrôle de gestion, audit',
        '28|101804': u'.....Direction gestion, finance',
        '29|101805': u'.....Droit, fiscalité',
        '30|101806': u'.....Finance, trésorerie',
        '31|101833': u'Informatique',
        '32|101807': u'.....Direction informatique',
        '33|101808': u'.....Exploitation, maintenance informatique',
        '34|101809': u'.....Informatique de gestion',
        '35|101810': u'.....Informatique industrielle',
        '36|101811': u'.....Informatique web, sites et portails Internet',
        '37|101812': u'.....Maîtrise d\'ouvrage et fonctionnel',
        '38|101813': u'.....Système, réseaux, données',
        '39|101834': u'Production Industrielle, Travaux, Chantiers',
        '40|101814': u'.....Cadres de chantier',
        '41|101815': u'.....Cadres de production industrielle',
        '42|101816': u'.....Direction d\'unité industrielle',
        '43|101835': u'Ressources Humaines',
        '44|101817': u'.....Administration des RH',
        '45|101818': u'.....Développement des RH',
        '46|101819': u'.....Direction des ressources humaines',
        '47|101820': u'.....Formation initiale et continue',
        '48|101836': u'Sanitaire, Social, Culture',
        '49|101821': u'.....Activités sanitaires, sociales et culturelles',
        '50|101837': u'Services Techniques',
        '51|101822': u'.....Achats',
        '52|101823': u'.....Direction des services techniques',
        '53|101824': u'.....Logistique',
        '54|101825': u'.....Maintenance, sécurité',
        '55|101826': u'.....Process, méthodes',
        '56|101827': u'.....Qualité',
    }.items())])

    secteur_choices = OrderedDict([(k, u'%s' % (v)) for k, v in sorted({
        ' ': u'-- Indifférent --',
        '101752': u'Activités des organisations associatives et administration publique',
        '101753': u'Activités informatiques',
        '101754': u'Activités juridiques et comptables',
        '101755': u'Agroalimentaire',
        '101756': u'Automobile, aéronautique et autres matériels de transport',
        '101757': u'Banque et Assurances',
        '101758': u'Bois - Papier - Imprimerie',
        '101759': u'Chimie - Caoutchouc - Plastique',
        '101760': u'Commerce interentreprises',
        '101761': u'Communication et médias',
        '101762': u'Conseil et gestion des entreprises',
        '101763': u'Construction',
        '101764': u'Distribution généraliste et spécialisée',
        '101765': u'Energies - Eau',
        '101766': u'Equipements électriques et électroniques',
        '101767': u'Formation initiale et continue',
        '101768': u'Gestion des déchets',
        '101769': u'Hôtellerie - Restauration - Loisirs',
        '101770': u'Immobilier',
        '101771': u'Industrie pharmaceutique',
        '101772': u'Ingénierie - R et D',
        '101773': u'Intermédiaires du recrutement',
        '101774': u'Mécanique - Métallurgie',
        '101775': u'Meuble, Textile et autres industries manufacturières',
        '101776': u'Santé - action sociale',
        '101777': u'Services divers aux entreprises',
        '101778': u'Télécommunications',
        '101779': u'Transports et logistique',
    }.items())])

    type_contrat_choices = OrderedDict([(k, u'%s' % (v)) for k, v in sorted(APEC_CONTRATS.items())])

    salary_choices = OrderedDict([(k, u'%s' % (v)) for k, v in sorted({
        ' ': u'-- Indifférent --',
        '0|35': u'Moins de 35 K€',
        '35|50': u'Entre 35 et 49 K€',
        '50|70': u'Entre 50 et 69 K€',
        '70|90': u'Entre 70 et 90 K€',
        '90|1000': u'Plus de 90 K€',
    }.items())])

    date_choices = OrderedDict([(k, u'%s' % (v)) for k, v in sorted({
        ' ': u'-- Indifférent --',
        '101850': u'Aujourd\'hui',
        '101851': u'Les 7 derniers jours',
        '101852': u'Les 30 derniers jours',
        '101853': u'Toutes les offres',
    }.items())])

    level_choices = OrderedDict([(k, u'%s' % (v)) for k, v in sorted(APEC_EXPERIENCE.items())])

    CONFIG = BackendConfig(Value('place', label=u'Lieu', choices=places_choices, default=''),
                           Value('fonction', label=u'Fonction', choices=fonction_choices, default=''),
                           Value('secteur', label=u'Secteur', choices=secteur_choices, default=''),
                           Value('contrat', label=u'Contrat', choices=type_contrat_choices, default=''),
                           Value('salaire', label=u'Salaire', choices=salary_choices, default=''),
                           Value('limit_date', label=u'Date', choices=date_choices, default=''),
                           Value('level', label=u'Expérience', choices=level_choices, default=''))

    def search_job(self, pattern=None):
        for job_advert in self.browser.search_job(pattern=pattern):
            yield self.fill_obj(job_advert)

    def decode_choice(self, choice):
        splitted_choice = choice.split('|')
        if len(splitted_choice) == 2:
            return splitted_choice[1]
        else:
            return ''

    def advanced_search_job(self):
        for job_advert in self.browser.advanced_search_job(region=self.decode_choice(self.config['place'].get()),
                                                           fonction=self.decode_choice(self.config['fonction'].get()),
                                                           secteur=self.config['secteur'].get(),
                                                           salaire=self.config['salaire'].get(),
                                                           contrat=self.config['contrat'].get(),
                                                           limit_date=self.config['limit_date'].get(),
                                                           level=self.config['level'].get()):
            yield self.fill_obj(job_advert)

    def get_job_advert(self, _id, advert=None):
        job_advert = self.browser.get_job_advert(_id, advert)
        return self.fill_obj(job_advert)

    def fill_obj(self, advert, fields=None):
        if advert.contract_type in self.type_contrat_choices:
            advert.contract_type = self.type_contrat_choices[advert.contract_type]

        if advert.experience in self.level_choices:
            advert.experience = self.level_choices[advert.experience]

        return advert

    OBJECTS = {BaseJobAdvert: fill_obj}



# -*- coding: utf-8 -*-

# Copyright(C) 2010-2011 Christophe Benz
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

import logging
import time

__all__ = ['retry']


def retry(ExceptionToCheck, tries=4, delay=3, backoff=2):
    """
    Retry decorator
    from http://www.saltycrane.com/blog/2009/11/trying-out-retry-decorator-python/
    original from http://wiki.python.org/moin/PythonDecoratorLibrary#Retry
    """
    def deco_retry(f):
        def f_retry(*args, **kwargs):
            mtries, mdelay = tries, delay
            mtries = kwargs.pop('_tries', mtries)
            mdelay = kwargs.pop('_delay', mdelay)
            try_one_last_time = True
            while mtries > 1:
                try:
                    return f(*args, **kwargs)
                    try_one_last_time = False
                    break
                except ExceptionToCheck as e:
                    try:
                        logging.debug(u'%s, Retrying in %d seconds...' % (e, mdelay))
                    except UnicodeDecodeError:
                        logging.debug(u'%s, Retrying in %d seconds...' % (repr(e), mdelay))
                    time.sleep(mdelay)
                    mtries -= 1
                    mdelay *= backoff
            if try_one_last_time:
                return f(*args, **kwargs)
            return
        return f_retry  # true decorator
    return deco_retry
